#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : common
# @Time         : 2025/3/23 11:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import shortuuid

from meutils.pipe import *
from meutils.io.files_utils import to_bytes, to_url
from meutils.caches import rcache

from meutils.schemas.textin_types import BASE_URL, WatermarkRemove, PdfToMarkdown, CropEnhanceImage

from httpx import AsyncClient


class Textin(object):
    def __init__(self, api_key: Optional[str] = None):
        # https://www.textin.com/console/dashboard/setting
        app_id, secret_code = (api_key or os.getenv("TEXTIN_API_KEY")).split("|")

        logger.debug(f"{app_id, secret_code}")

        self.base_url = BASE_URL
        self.headers = {
            'x-ti-app-id': app_id,
            'x-ti-secret-code': secret_code,
            'Content-Type': "text/plain"
        }

    # @rcache(noself=True, ttl=24 * 3600, serializer="pickle")
    async def image_watermark_remove(self, request: WatermarkRemove):
        s = time.perf_counter()

        if not request.image.startswith("http"):
            request.image = await to_bytes(request.image)
            # content_type = "application/octet-stream"
            # logger.info(f"image: {type(request.image)}")

        async with AsyncClient(base_url=self.base_url, headers=self.headers, timeout=100) as cilent:
            response = await cilent.post("/image/watermark_remove", content=request.image)
            response.raise_for_status()

            data = response.json()
            data['timings'] = {'inference': time.perf_counter() - s}

            logger.debug(data)

            if request.response_format == "url" and data.get("code") == 200:
                data["result"]["image"] = await to_url(data["result"]["image"], filename=f'{shortuuid.random()}.png')

            return data

    async def pdf_to_markdown(self, request: PdfToMarkdown, params: Optional[dict] = None):  # crop_enhance_image
        s = time.perf_counter()

        request.data = await to_bytes(request.data)

        params = params or {}
        async with AsyncClient(base_url=self.base_url, headers=self.headers, timeout=100, params=params) as cilent:
            response = await cilent.post("pdf_to_markdown", content=request.data)
            response.raise_for_status()

            data = response.json()
            data['timings'] = {'inference': time.perf_counter() - s}

            logger.debug(bjson(data))

            return data

    async def crop_enhance_image(self, request: CropEnhanceImage, params: Optional[dict] = None):  # crop_enhance_image
        s = time.perf_counter()

        request.data = await to_bytes(request.data)

        params = params or {}
        async with AsyncClient(base_url=self.base_url, headers=self.headers, timeout=100, params=params) as cilent:
            response = await cilent.post("crop_enhance_image", content=request.data)
            response.raise_for_status()

            data = response.json()
            data['timings'] = {'inference': time.perf_counter() - s}

            # {'code': 200,
            #  'duration': 161,
            #  'message': 'success',
            #  'result': {'image_list': [{'angle': 0,
            #                             'cropped_height': 848,
            #                             'cropped_width': 628,
            #                             'image':

            if request.response_format == "url" and data.get("code") == 200:
                for image in data["result"]["image_list"]:
                    image["image"] = await to_url(image["image"], filename=f'{shortuuid.random()}.png')

            logger.debug(bjson(data))

            return data


if __name__ == '__main__':
    # image = "doc_watermark.jpg"

    # image = "https://oss.ffire.cc/files/nsfw.jpg"
    image = "https://oss.ffire.cc/files/kling_watermark.png"  # 无水印

    request = WatermarkRemove(
        image=image,
        response_format="url"
    )
    # arun(Textin().image_watermark_remove(request))

    request = PdfToMarkdown(
        data="https://s3.ffire.cc/files/pdf_to_markdown.jpg",
        response_format="url"
    )

    # arun(Textin().pdf_to_markdown(request))

    request = CropEnhanceImage(
        data="https://s3.ffire.cc/files/pdf_to_markdown.jpg",
        response_format="url"
    )

    arun(Textin().crop_enhance_image(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : pdf
# @Time         : 2023/5/18 16:39
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *


class PageWord(BaseModel):
    x0 = 153.5
    x1 = 441.72032
    top = 76.19035999999994
    doctop = 76.19035999999994
    bottom = 92.15035999999998
    upright = True
    direction = 1
    text = "国投瑞银基金管理有限公司基金相关参数"


class PageWords(BaseModel):  # page.extract_words()
    pagewords: List[PageWord]


class Part(BaseModel):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : adapters
# @Time         : 2025/5/30 16:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import shortuuid
from aiostream import stream

from meutils.pipe import *
from meutils.io.files_utils import to_url, to_url_fal
from meutils.str_utils.json_utils import json_path
from meutils.llm.openai_utils import create_chat_completion
from meutils.schemas.openai_types import CompletionRequest, ChatCompletion
from meutils.schemas.image_types import ImageRequest, ImagesResponse
from meutils.llm.openai_utils import chat_completion, chat_completion_chunk, create_chat_completion_chunk
from meutils.str_utils import parse_url, parse_command_string


async def stream_to_nostream(
        request: CompletionRequest,
):
    pass


async def chat_for_image(
        generate: Optional[Callable],
        request: CompletionRequest,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
):
    generate = generate and partial(generate, api_key=api_key, base_url=base_url)

    image = None
    prompt = request.last_user_content
    if image_urls := request.last_urls.get("image_url"):  # image_url
        if request.model.startswith('fal'):
            urls = await to_url_fal(image_urls, content_type="image/png")  # 国外友好
            image = urls

        elif request.model.startswith("doubao-seed"):
            image = image_urls  # b64

        else:
            urls = await to_url(image_urls, ".png", content_type="image/png")  # 数组
            image = urls

    image_request = ImageRequest(
        model=request.model,
        prompt=prompt,
        image=image
    )
    if not image_request.image:
        image_request.image, image_request.prompt = image_request.image_and_prompt

    if '--' in image_request.prompt:
        prompt_dict = parse_command_string(image_request.prompt)
        # 缩写补充
        prompt_dict['aspect_ratio'] = prompt_dict.get('aspect_ratio') or prompt_dict.get('ar')

        data = {
            **image_request.model_dump(exclude_none=True, exclude={"extra_fields", "aspect_ratio"}),
            **prompt_dict
        }
        image_request = ImageRequest(**data)
        logger.debug(image_request)

    # 非流式
    if not request.stream:
        if request.last_user_content.startswith(  # 跳过nextchat
                (
                        "hi",
                        "使用四到五个字直接返回这句话的简要主题",
                        "简要总结一下对话内容，用作后续的上下文提示 prompt，控制在 200 字以内"
                )):
            return chat_completion

        response = await generate(image_request)  # None

        if not response:
            raise Exception(f"image generate error: {str(request)[:1000]}")

        if not isinstance(response, dict):
            response = response.model_dump()

        content = ""
        for image in response['data']:
            content += f"""![{image.get("revised_prompt")}]({image['url']})\n\n"""

        chat_completion.choices[0].message.content = content
        return chat_completion

    # 流式
    if not generate: return

    future_task = asyncio.create_task(generate(image_request))  # 异步执行

    async def gen():
        exclude = None
        if len(str(image_request.image)) > 1000:
            exclude = {"image"}

        text = image_request.model_dump_json(exclude_none=True, exclude=exclude).replace("free", "")
        for i in f"""> 🖌️正在绘画\n\n```json\n{text}\n```\n\n""":
            await asyncio.sleep(0.05)
            yield i

        try:
            response = await future_task
            # response = await response  # 注意

            if not isinstance(response, dict):
                response = response.model_dump()

            for image in response['data']:
                yield f"""![{image.get("revised_prompt")}]({image['url']})\n\n"""


        except Exception as e:
            # yield f"```error\n{e}\n```\n"
            raise e

    chunks = create_chat_completion_chunk(gen(), redirect_model=request.model)
    return chunks


async def chat_for_video(
        get_task: Callable,  # response
        taskid: str,
):
    """异步任务"""

    async def gen():

        # 获取任务
        for i in f"""> VideoTask(id={taskid})\n""":
            await asyncio.sleep(0.03)
            yield i

        yield f"[🤫 任务进度]("
        for i in range(60):
            await asyncio.sleep(3)
            response = await get_task(taskid)  # 包含  "status"

            logger.debug(response)
            if response.get("status", "").lower().startswith(("succ", "fail")):

                yield ")🎉🎉🎉\n\n"

                yield f"""```json\n{json.dumps(response, indent=4, ensure_ascii=False)}\n```"""

                if urls := json_path(response, expr='$..[url,image_url,video_url]'):  # 所有url
                    for i, url in enumerate(urls, 1):
                        yield f"\n\n[下载链接{i}]({url})\n\n"

                break

            else:
                yield "🚀"

    chunks = create_chat_completion_chunk(gen(), chat_id=taskid)
    return chunks


if __name__ == '__main__':
    from meutils.apis.images.generations import generate

    request = CompletionRequest(
        model="deepseek-r1-Distill-Qwen-1.5B",
        messages=[
            {"role": "user", "content": "``hi --a 1"}
        ],
        stream=True,
    )
    arun(chat_for_image(None, request))

    request = CompletionRequest(
        model="gemini-2.5-flash-image-preview",
        messages=[
            {"role": "user", "content": "画条狗"}
        ],
        # stream=True,
    )
    api_key = "sk-MAZ6SELJVtGNX6jgIcZBKuttsRibaDlAskFAnR7WD6PBSN6M-openai"
    base_url = "https://new.yunai.link/v1"
    arun(chat_for_image(generate, request, api_key, base_url))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : wecom
# @Time         : 2021/8/31 下午5:16
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : https://github.com/quanttide/wecom-sdk-py/tree/master/wechatwork_sdk


from meutils.pipe import *
from meutils.hash_utils import *


class Article(BaseConfig):
    title: str = '百度热榜'
    description: str = '百度热榜'
    url: str = 'https://top.baidu.com/board?tab=realtime'
    picurl: str = 'http://res.mail.qq.com/node/ww/wwopenmng/images/independent/doc/test_pic_msg1.png'


class Wecom(object):

    def __init__(self, hook_url=None):
        self.hook_url = self._convert_hook_url(hook_url)

    def send_markdown(self, title="", content=""):

        if isinstance(content, (list, tuple)):
            content = '\n'.join(content).strip()

        json = {
            "msgtype": "markdown",
            "markdown": {
                "content": f"# {title}\n{content}".strip()
            }
        }
        return self._send(json)

    def send_text(self, title="", content="", mentioned_mobile_list=None):
        json = {
            "msgtype": "text",
            "text": {
                "content": f"{title}\n{content}".strip()
            }
        }
        if mentioned_mobile_list:  # "@all"'
            json['text']['mentioned_mobile_list'] = mentioned_mobile_list

        return self._send(json)

    def send_news(self, articles: Union[Article, List[Article]]):
        """
        json = {
        "msgtype": "news",
        "news": {
        "articles": [
            {
                "title": "中秋节礼品领取1",
                "description": "今年中秋节公司有豪礼相送",  # 多条不会显示
                "url": "www.qq.com",
                "picurl": "http://res.mail.qq.com/node/ww/wwopenmng/images/independent/doc/test_pic_msg1.png"
            }
        ]}}
        """
        json = {
            "msgtype": "news",
            "news": {
                "articles": []
            }
        }
        if isinstance(articles, Article):
            articles = [articles]

        for article in articles:
            json["news"]["articles"].append(article.dict())
        return self._send(json)

    def send_card(self, ):
        json = {
            "msgtype": "template_card",
            "template_card": {
                "card_type": "news_notice",
                "source": {
                    "icon_url": "https://wework.qpic.cn/wwpic/252813_jOfDHtcISzuodLa_1629280209/0",
                    "desc": "企业微信"
                },
                "main_title": {
                    "title": "欢迎使用企业微信",
                    "desc": "您的好友正在邀请您加入企业微信"
                },
                "card_image": {
                    "url": "https://wework.qpic.cn/wwpic/354393_4zpkKXd7SrGMvfg_1629280616/0",
                    "aspect_ratio": 2.25
                },
                "vertical_content_list": [
                    {
                        "title": "惊喜红包等你来拿",
                        "desc": "下载企业微信还能抢红包！"
                    }
                ],
                "horizontal_content_list": [
                    {
                        "keyname": "邀请人",
                        "value": "张三"
                    },
                    {
                        "keyname": "企微官网",
                        "value": "点击访问",
                        "type": 1,
                        "url": "https://work.weixin.qq.com/?from=openApi"
                    },
                    {
                        "keyname": "企微下载",
                        "value": "企业微信.apk",
                        "type": 2,
                        "media_id": "MEDIAID"
                    }
                ],
                "jump_list": [
                    {
                        "type": 1,
                        "url": "https://work.weixin.qq.com/?from=openApi",
                        "title": "企业微信官网"
                    },
                    {
                        "type": 2,
                        "appid": "APPID",
                        "pagepath": "PAGEPATH",
                        "title": "跳转小程序"
                    }
                ],
                "card_action": {
                    "type": 1,
                    "url": "https://work.weixin.qq.com/?from=openApi",
                    "appid": "APPID",
                    "pagepath": "PAGEPATH"
                }
            }
        }

        self._send(json)

    def send_image(self, path='http://www.nesc.cn/dbzq/images/logo.png'):
        bytes_data = self._get_bytes(path)
        body = {
            "msgtype": "image",
            "image": {
                "base64": bytes2base64(bytes_data),
                "md5": md5(bytes_data, False)
            }
        }
        return self._send(body)

    def send_file(self, path, type='file'):
        """

        @param path:
        @param type: 要求文件大小在5B~20M之间，媒体文件类型，分别有图片（image）、语音（voice）、视频（video），普通文件(file)
        @return:
        """

        upload_media_url = f"""{self.hook_url.replace('send?', 'upload_media?')}&type={type}"""

        # bytes_data = self._get_bytes(path)
        # files = {'data': bytes_data} # 这样文件名为data

        with open(path, 'rb') as f:
            files = {'data': f}
            response = requests.post(upload_media_url, files=files)
        try:
            media_id = response.json()['media_id']
            body = {"msgtype": type, type: {"media_id": media_id}}
            return self._send(body)

        except Exception as e:
            return response.json()

    def _send(self, body=None):
        if body is None:
            body = {
                "msgtype": "text",
                "text": {
                    "content": "南京今日天气：29度，大部分多云，降雨概率：60%",
                    "mentioned_mobile_list": ["18550288233", "@all"]
                }
            }
        return requests.post(url=self.hook_url, json=body)

    @staticmethod
    def _get_bytes(path):
        if path.startswith('http'):
            bytes_data = requests.get(path).content
        else:
            bytes_data = Path(path).read_bytes()
        return bytes_data

    @staticmethod
    def _convert_hook_url(hook_url):
        if hook_url is None or hook_url == '':
            hook_url = '43e28f3b-0c07-492c-aa37-a59abc3acf43'

        if not hook_url.startswith('http'):
            hook_url = f'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key={hook_url}'
        return hook_url


if __name__ == '__main__':
    wecom = Wecom()
    # wecom.send_news(Article())
    # wecom.send_file('wecom.py')
    wecom.send_card()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2023/10/18 13:57
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import time

from meutils.pipe import *


class A(BaseModel):
    a: Literal['a', 'b']


import streamlit as st
from streamlit_extras.streaming_write import write

_LOREM_IPSUM = """
Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut
labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco
laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in
voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat
non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.
"""


def stream_example():
    for word in _LOREM_IPSUM.split():
        yield word + " "
        time.sleep(0.1)

    # # Also supports any other object supported by `st.write`
    # yield pd.DataFrame(
    #     np.random.randn(5, 10),
    #     columns=["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"],
    # )
    #
    # for word in _LOREM_IPSUM.split():
    #     yield word + " "
    #     time.sleep(0.05)


def f():
    with st.chat_message('assistant'):

        write('😘😘😘 嗨，我是你的解题小能手！\n\n **参考示例**：')






#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : firecrawl
# @Time         : 2024/7/29 10:07
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://www.firecrawl.dev/pricing

from meutils.pipe import *
import requests
import json

url = "https://api.firecrawl.dev/v0/scrape"


async def scrape_url(url):
    payload = {
        "url": "https://mp.weixin.qq.com/s/kFJS7sk66V8Eq0hEwJWw7g",
        "crawlerOptions": {
            "generateImgAltText": False
        },
        "pageOptions": {
            "onlyMainContent": False,
            "removeTags": [
                ""
            ],
            "onlyIncludeTags": [
                ""
            ]
        },
        "origin": "website-preview"
    }
    headers = {
        'authorization': 'Bearer this_is_just_a_preview_token',
    }
    async with httpx.AsyncClient(headers=headers, timeout=60) as client:
        response = await client.post(url=url, json=payload)
        if response.is_success:
            return response.json()


if __name__ == '__main__':
    arun(scrape_url(url))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : textin
# @Time         : 2024/6/26 08:22
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo: 重构 https://tools.textin.com/
# https://www.textin.com/console/recognition/robot_enhance?service=watermark-remove

# proxies 取消了 https://github.com/encode/httpx/pull/2879

from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.notice.feishu import send_message
from meutils.io.files_utils import to_url_fal as to_url
from meutils.apis.proxy.kdlapi import get_one_proxy

from fake_useragent import UserAgent

ua = UserAgent()

BASE_URL = "https://api.textin.com/home"

proxy = "http://110.42.51.201:38443"


@alru_cache(ttl=3600 * 24)
async def document_process(data: bytes,
                           service: str = "pdf_to_markdown",
                           response_format: str = "url",
                           token: Optional[str] = None,  # 821480df15b637852f7e4a0dabb8a156
                           **kwargs):
    """

    :param data:
    :param service: pdf_to_markdown watermark-remove dewarp
    :param response_format:
    :param kwargs:
    :return:
    """

    params = {
        "service": service,
        **kwargs

        # "page_count": page_count,
        # "get_image": "objects"
        # "apply_document_tree": 0,
    }

    request_kwargs = {}
    # request_kwargs = {"proxies": proxies}

    for i in range(3):

        headers = {
            "token": token or str(uuid.uuid4()).replace("-", ""),
            'User-Agent': ua.random,
        }
        try:
            async with httpx.AsyncClient(base_url=BASE_URL, timeout=60, headers=headers, **request_kwargs) as client:
                response = await client.post('/user_trial_ocr', content=data, params=params)
                response.raise_for_status()

                logger.debug(response.status_code)
                # logger.info(response.json())  # {'msg': '今日请求超过限制次数', 'code': 431}

                _ = response.json()

                if '今日请求超过限制次数' in str(_):
                    # logger.info(response.json())
                    raise Exception("更新请求头或者加代理")
                else:
                    break


        except Exception as e:  # 加代理
            logger.error(e)

            request_kwargs = {
                "proxy": await get_one_proxy(),
                # "proxy": proxy,

            }

    if response_format == "url" and service in {"watermark-remove", "crop_enhance_image"}:
        if "data" not in _:
            send_message(_, title=__name__)

        # _ => {'msg': '今日请求超过限制次数', 'code': 431}
        logger.debug(bjson(_))
        # logger.debug(list(_['data']['result']['image']))

        url = await to_url(_['data']['result']['image'])
        _ = {"data": [{"url": url}]}

        # _['data']['result']['image'] = await to_url(_['data']['result']['image'])

    return _


if __name__ == '__main__':
    from meutils.io.files_utils import to_bytes, to_url

    # data = open("/Users/betterme/PycharmProjects/AI/11.jpg", 'rb').read()
    # # data = open("/Users/betterme/PycharmProjects/AI/蚂蚁集团招股书.pdf", 'rb').read()
    # with timer("解析"):
    #     # arun(textin_fileparser(data))
    #     print(arun(textin_fileparser(data, service="pdf_to_markdown")))

    # response = requests.request("POST", url, data=data)
    data = open("/Users/betterme/PycharmProjects/AI/qun.png", 'rb').read()
    data = open("img.png", 'rb').read()
    # data = arun(to_bytes("https://cdn.meimeiqushuiyin.cn/ori/tmp_e8d6329e1b2c1bc541ca530fcbae14e3ec12f65d8d4ec97d.jpg"))

    from meutils.schemas.task_types import Purpose

    service = Purpose.watermark_remove.value

    # service = "pdf_to_markdown"

    service = 'bill_recognize_v2'


    async def main(n=1):
        for i in tqdm(range(n)):
            try:
                _ = await document_process(data, service=service)
                logger.debug(bjson(_))
            except Exception as e:
                logger.error(f"{i}: {e}")
                break


    # with timer("解析"):
    #
    #     # arun(textin_fileparser(data))
    #     data = arun(document_process(data, service=service))
    #
    #     # b64 = data['data']['result']['image']
    #
    #     # base64_to_file(b64, "demo.png")
    #
    #     # data['data']['result']['image'] = arun(to_url(b64))
    #
    #     logger.debug(data)
    #
    #     # {'msg': 'success',
    #     #  'data': {
    #     #      'result': {
    #     #          'image': 'https://sfile.chatglm.cn/chatglm-videoserver/image/e5/e5d4011c.png'
    #     #      },
    #     #      'file_type': '', 'file_data': ''
    #     #  }, 'code': 200
    #     #  }

    arun(main(1))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ts
# @Time         : 2024/4/29 10:40
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : translators
import os

import httpx

from meutils.pipe import *

import translators as ts

print(ts.translators_pool)

'niutrans'

# r = ts.translate_text(
#     query_text='你好',
#     translator='niutrans',
#     to_language='en',
#     apikey='f77c833dc48cf93e1e85bea2d6f17459'
#     # **kwargs: ApiKwargsType,
# )


if __name__ == '__main__':
    headers = {
        "Cookie": os.getenv("PAY_COOKIE"),
        "Content-Type": "application/x-www-form-urlencoded"
    }

    payload = {
        # "sitename": "",
        "trade_no": 2024042915220175845,
    }

    # print(httpx.post("https://zf.96ym.cn/User/Order/Callback", headers=headers, data=payload).text)
    # html = httpx.get("https://zf.96ym.cn/User/Order/Index", headers=headers).text
    # # print(html)
    #
    # print(pd.read_html(html)[0].query("状态!='已完成' and 商品名称!='测试通道'"))

    df = pd.read_html("https://zf.96ym.cn/User/Order/Index", storage_options=headers)[0]
    df = df.query(f"`状态`!='已完成' and `商品名称`!='测试通道' and `创建时间 支付时间`>'{str(datetime.datetime.now())[:10]} 00:00:00'")

    print(df)

    # """
    # import httpx; httpx.post("https://zf.96ym.cn/User/Order/Callback", headers=headers, data=payload).text
    #
    # """







#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : retry
# @Time         : 2021/3/18 2:57 下午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from tenacity import retry, stop_after_attempt, before_sleep_log
from tenacity import wait_fixed, wait_exponential, wait_incrementing, wait_exponential_jitter  # Wait strategy
from tenacity import retry_if_result, RetryCallState  # 重试策略
from tenacity import retry_if_exception, retry_if_exception_type, retry_if_not_exception_type

# 不兼容异步
from meutils.notice.feishu import send_message


class IgnoredRetryException(Exception):
    pass


def default_retry_error_callback(retry_state: RetryCallState, title: Optional[str] = None):
    """return the result of the last call attempt"""
    # logger.debug(f"最后一次重试仍然失败调用的函数: {retry_state.outcome}")
    logger.debug(f"最后一次重试仍然失败调用的函数: {retry_state}")

    send_message(f"""{retry_state}""", title)

    # logger.debug(retry_state.outcome.result())

    exc = retry_state.outcome.exception()  # 最后抛出错误
    if exc:
        raise exc  # 最后返回结果


def not_in_status_codes(exception):
    """451不触发重试"""
    # logger.debug(exception)

    if isinstance(exception, httpx.HTTPStatusError):
        # logger.debug(f"status_code: {exception.response.status_code}")

        status_code = exception.response.status_code
        return status_code not in {451}
    return True


def \
        retrying(
        max_retries=2,
        exp_base=2,
        min: int = 0,
        max: int = 100000,
        retry_error_callback: Optional[Callable[[RetryCallState], Any]] = None,
        predicate: Callable[[Any], bool] = lambda r: False,
        title: Optional[str] = None,
        ignored_exception_types: Optional[typing.Union[
            typing.Type[BaseException],
            typing.Tuple[typing.Type[BaseException], ...],
        ]] = None,  # 该错误类型不重试
):
    import logging

    logger = logging.getLogger()

    retry_error_callback = retry_error_callback or partial(default_retry_error_callback, title=title)

    # 抛错或者返回结果判断为True重试 就重试
    retry_reasons = (
            (
                    retry_if_exception_type()
                    & retry_if_exception(not_in_status_codes)
            )
            | retry_if_result(predicate)
    )
    if ignored_exception_types:
        retry_reasons = retry_reasons & retry_if_not_exception_type(ignored_exception_types)

    _retry_decorator = retry(
        reraise=True,
        stop=stop_after_attempt(max_retries),
        wait=wait_exponential(multiplier=1.0, exp_base=exp_base, min=min, max=max),  # max=max_seconds
        retry=retry_reasons,
        # before_sleep=before_sleep_log(logger, 30),
        retry_error_callback=retry_error_callback,
    )

    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        return _retry_decorator(wrapped)(*args, **kwargs)

    return wrapper


def create_retry_decorator() -> Callable[[Any], Any]:  # todo: Retrying
    """
    @create_retry_decorator()
    def fn():
        pass

    :return:
    """
    import openai
    max_retries = 3
    min_seconds = 4
    max_seconds = 10
    # Wait 2^x * 1 second between each retry starting with
    # 4 seconds, then up to 10 seconds, then 10 seconds afterwards
    return retry(
        reraise=True,
        stop=stop_after_attempt(max_retries),
        wait=wait_exponential(multiplier=1, min=min_seconds, max=max_seconds),
        retry=(
                retry_if_exception_type(openai.error.Timeout)
                | retry_if_exception_type(openai.error.APIError)
                | retry_if_exception_type(openai.error.APIConnectionError)
                | retry_if_exception_type(openai.error.RateLimitError)
                | retry_if_exception_type(openai.error.ServiceUnavailableError)
        ),
        before_sleep=before_sleep_log(logger, 30),
    )


def wait_retry(n=3):
    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        @retry(wait=wait_fixed(n))
        def wait():
            logger.warning("retry")
            if wrapped(*args, **kwargs):  # 知道检测到True终止
                return True

            raise Exception

        return wait()

    return wrapper


# from meutils.cmds import HDFS
# HDFS.check_path_isexist()


if __name__ == '__main__':
    # s = time.time()  # 1616145296
    # print(s)
    # e1 = s + 10
    # e2 = e1 + 10
    #
    #
    # @wait_retry(5)
    # def f(e):
    #     return time.time() > e  # 变的
    #
    #
    # def run(e):
    #     f(e)
    #     print(f"task {e}")
    #
    #
    # # for e in [e2, e1]:
    # #     print(run(e))
    # #
    # # print("耗时", time.time() - s)
    #
    # [e1, e2, 1000000000000] | xProcessPoolExecutor(run, 2)

    # class retry_state:
    #     attempt_number = 0
    #
    #
    # for i in range(1, 10):
    #     retry_state.attempt_number = i
    #     # print(wait_incrementing(100, -10)(retry_state))
    #     print(wait_exponential(1, exp_base=2)(retry_state))

    # @retrying(3)
    # async def f():
    #     logger.debug("retry")
    #     pass
    #     # 1 / 0
    #     raise Exception('xx')

    # try:
    #     f()
    #
    # except Exception as e:
    #     print(e)
    #     print(type(e))

    # @retrying(3)
    # def f():
    #     # async def f():
    #     logger.debug("retry")
    #     for i in range(10):
    #         yield ""
    #         if i == 8:
    #             raise Exception('xx')

    # arun(f())

    # f()

    # 示例使用
    class CustomError1(Exception):
        pass


    class CustomError2(Exception):
        pass


    @retrying(
        max_retries=3,
        ignored_exception_types=(CustomError1, IgnoredRetryException)
    )
    def my_function():
        logger.debug(1)
        # 模拟抛出异常
        raise IgnoredRetryException("This is a custom error")


    def main():
        try:
            my_function()
        except Exception as e:
            print(f"Operation failed after all retries: {e}")


    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : modelscope
# @Time         : 2025/8/22 22:47
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from openai import OpenAI

models_mapping = {
    "flux-kontext-dev": "MusePublic/FLUX.1-Kontext-Dev",
    # "flux-kontext-dev": "black-forest-labs/FLUX.1-Kontext-dev",

    "flux.1-krea-dev": "black-forest-labs/FLUX.1-Krea-dev",

    "kimi-k2-0711-preview": "moonshotai/Kimi-K2-Instruct",
    "kimi-k2-250711": "moonshotai/Kimi-K2-Instruct",
    "kimi-k2-0905-preview": "moonshotai/Kimi-K2-Instruct-0905",
    "kimi-k2-250905": "moonshotai/Kimi-K2-Instruct-0905",

    "majicflus_v1": "MAILAND/majicflus_v1",
    "deepseek-reasoner": "deepseek-ai/DeepSeek-R1-0528",

    "deepseek-r1": "deepseek-ai/DeepSeek-R1-0528",
    "deepseek-r1-0528": "deepseek-ai/DeepSeek-R1-0528",
    "deepseek-r1-250528": "deepseek-ai/DeepSeek-R1-0528",
    "deepseek-chat": "deepseek-ai/DeepSeek-V3",
    "deepseek-v3": "deepseek-ai/DeepSeek-V3",
    "deepseek-v3-0324": "deepseek-ai/DeepSeek-V3",
    "deepseek-v3-250324": "deepseek-ai/DeepSeek-V3",
    "deepseek-v3-1-250821": "deepseek-ai/DeepSeek-V3.1",

    "deepseek-r1-distill-qwen-14b": "deepseek-ai/DeepSeek-R1-Distill-Qwen-14B",
    "deepseek-r1-distill-qwen-32b": "deepseek-ai/DeepSeek-R1-Distill-Qwen-32B",
    "deepseek-r1-distill-llama-70b": "deepseek-ai/DeepSeek-R1-Distill-Llama-70B",
    "qwen2.5-coder-32b-instruct": "Qwen/Qwen2.5-Coder-32B-Instruct",
    "qwen2.5-coder-14b-instruct": "Qwen/Qwen2.5-Coder-14B-Instruct",
    "qwen2.5-coder-7b-instruct": "Qwen/Qwen2.5-Coder-7B-Instruct",
    "qwen2.5-72b-instruct": "Qwen/Qwen2.5-72B-Instruct",
    "qwen2.5-32b-instruct": "Qwen/Qwen2.5-32B-Instruct",
    "qwen2.5-14b-instruct": "Qwen/Qwen2.5-14B-Instruct",
    "qwen2.5-7b-instruct": "Qwen/Qwen2.5-7B-Instruct",
    "qwq-32b-preview": "Qwen/QwQ-32B-Preview",
    "qvq-72b-preview": "Qwen/QVQ-72B-Preview",
    "qwen2-vl-7b-instruct": "Qwen/Qwen2-VL-7B-Instruct",
    "qwen2.5-14b-instruct-1m": "Qwen/Qwen2.5-14B-Instruct-1M",
    "qwen2.5-7b-instruct-1m": "Qwen/Qwen2.5-7B-Instruct-1M",
    "qwen2.5-vl-3b-instruct": "Qwen/Qwen2.5-VL-3B-Instruct",
    "qwen2.5-vl-7b-instruct": "Qwen/Qwen2.5-VL-7B-Instruct",
    "qwen2.5-vl-72b-instruct": "Qwen/Qwen2.5-VL-72B-Instruct",
    "qwq-32b": "Qwen/QwQ-32B",
    "qwen2.5-vl-32b-instruct": "Qwen/Qwen2.5-VL-32B-Instruct",
    "qwen3-0.6b": "Qwen/Qwen3-0.6B",
    "qwen3-1.7b": "Qwen/Qwen3-1.7B",
    "qwen3-4b": "Qwen/Qwen3-4B",
    "qwen3-14b": "Qwen/Qwen3-14B",
    "qwen3-30b-a3b": "Qwen/Qwen3-30B-A3B",
    "qwen3-32b": "Qwen/Qwen3-32B",
    "qwen3-235b-a22b": "Qwen/Qwen3-235B-A22B",
    "qwen3-coder-480b-a35b-instruct": "Qwen/Qwen3-Coder-480B-A35B-Instruct",
    "qwen3-235b-a22b-instruct-2507": "Qwen/Qwen3-235B-A22B-Instruct-2507",

    "qwen3-vl-plus": "Qwen/Qwen3-VL-235B-A22B-Instruct",
    "qwen3-vl-235b-a22b": "Qwen/Qwen3-VL-235B-A22B-Instruct",

}


def get_models_mapping():
    client = OpenAI(
        api_key=os.getenv("MODELSCOPE_API_KEY"),
        base_url=os.getenv("MODELSCOPE_BASE_URL"),
    )

    models = client.models.list().data
    models = {
        m.id.split('/', maxsplit=1)[-1].lower(): m.id for m in models

        if any(i.lower() not in m.id.lower() for i in {"qwen-image", 'Kimi-K2'})
    }
    # logger.debug(models)
    return {**models, **models_mapping}


if __name__ == '__main__':
    models = get_models_mapping()
    print(bjson(models))
    print(','.join(models))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : plot_utils
# @Time         : 2020/11/25 1:49 下午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : https://blog.csdn.net/hiroyuu008/article/details/122013763

from meutils.pipe import *
import seaborn as sns

sns.set(style="darkgrid")  # darkgrid, whitegrid, dark, white,和ticks
sns.set_context('paper')


# sns.plotting_context()
# sns.axes_style()
# plt.style.use('ggplot')
# plt.style.use('seaborn')

def plot_cum_cr(s: pd.Series, bin_wide=100, bins=12, name=None):
    """Conversion rate
    累积转化率
    """
    l = []
    for i in range(1, bins + 1):
        l.append(s[:i * bin_wide].mean())

    pd.Series(l).to_frame(name).plot(marker='.', linestyle='-')
    return l

# https://www.cnblogs.com/Yang-Sen/p/11226005.html
# import cufflinks as cf
# cf.set_config_file(offline=True)



import random
from pyecharts import charts


class WordCloud(object):

    def __init__(self, data_pair, shape=None, width='900px', height='500px'):
        """
        ['circle', 'cardioid', 'diamond', 'triangle-forward', 'triangle', 'pentagon', 'star']
        wc.render()
        wc.render_notebook()

        """
        self._shapes = ['circle', 'cardioid', 'diamond', 'triangle-forward', 'triangle', 'pentagon', 'star']
        self.data_pair = data_pair
        self.shape = shape if shape else random.choice(self._shapes)
        self.width = width
        self.height = height

    @property
    def wc(self):
        wc = charts.WordCloud()
        wc.add("WordCloud", data_pair=self.data_pair, shape=self.shape)
        wc.width = self.width
        wc.height = self.height
        return wc


if __name__ == '__main__':
    pairs = [('中国', 33),
             ('苹果', 24),
             ('奚梦瑶', 20),
             ('美国', 16),
             ('特朗普', 16),
             ('何猷君', 15),
             ('戛纳', 13),
             ('红毯', 12),
             ('iPhone', 12),
             ('车队', 9),
             ('车祸', 9),
             ('优衣', 9),
             ('信息', 9),
             ('李亚鹏', 9),
             ('恋情', 9),
             ('任素', 9),
             ('男孩', 9),
             ('亚洲', 8),
             ('孩子', 8),
             ('大学生', 8)]

    WordCloud(pairs).wc.render_notebook()
    WordCloud(pairs).wc.render('x.html')





import lark_oapi as lark
from lark_oapi.api.sheets.v3 import *


# SDK 使用说明: https://github.com/larksuite/oapi-sdk-python#readme
def main():
    # 创建client
    # 使用 user_access_token 需开启 token 配置, 并在 request_option 中配置 token
    client = lark.Client.builder() \
        .enable_set_token(True) \
        .log_level(lark.LogLevel.DEBUG) \
        .build()



    # 构造请求对象
    request: FindSpreadsheetSheetRequest = FindSpreadsheetSheetRequest.builder() \
        .spreadsheet_token("APMRsoxW4hkIzwt693Bc9kWtnSd") \
        .sheet_id("95c9b1") \
        .request_body(Find.builder()
            .find_condition(FindCondition.builder()
                .range("PNIfrm!A1:C5")
                .match_case(True)
                .match_entire_cell(False)
                .search_by_regex(False)
                .include_formulas(False)
                .build())
            .find("hello")
            .build()) \
        .build()

    # 发起请求
    option = lark.RequestOption.builder().user_access_token("u-cyzn34LK53b8WEt6sihwnB1kiLCg1gDHra0011o00adz").build()
    response: FindSpreadsheetSheetResponse = client.sheets.v3.spreadsheet_sheet.find(request, option)

    # 处理失败返回
    if not response.success():
        lark.logger.error(
            f"client.sheets.v3.spreadsheet_sheet.find failed, code: {response.code}, msg: {response.msg}, log_id: {response.get_log_id()}")
        return

    # 处理业务结果
    lark.logger.info(lark.JSON.marshal(response.data, indent=4))


if __name__ == "__main__":
    main()

# curl --location --request GET 'https://open.feishu.cn/open-apis/sheets/v2/spreadsheets/APMRsoxW4hkIzwt693Bc9kWtnSd/values/8DkMAI?valueRenderOption=ToString&dateTimeRenderOption=FormattedString' \
# --header 'Authorization: Bearer t-ce3540c5f02ac074535f1f14d64fa90fa49621c0'


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : klingai
# @Time         : 2024/7/9 13:23
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import jsonpath

from meutils.pipe import *
from meutils.schemas import kling_types
from meutils.schemas.task_types import TaskType, Task
from meutils.schemas.kuaishou_types import BASE_URL, KlingaiVideoRequest, FEISHU_URL, FEISHU_URL_VIP

from meutils.decorators.retry import retrying
from meutils.notice.feishu import send_message as _send_message
from meutils.config_utils.lark_utils import get_next_token_for_polling

from meutils.apis.kuaishou.klingai import get_reward, get_point, check_token, download, get_task_plus

from meutils.apis.proxy.kdlapi import get_one_proxy

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)


# BASE_URL = GUOJI_BASE_URL


# 自动延长
# {"type":"m2v_extend_video","inputs":[{"name":"input","inputType":"URL","url":"https://h1.inkwai.com/bs2/upload-ylab-stunt/special-effect/output/HB1_PROD_ai_web_29545092/8992112608804666920/output_ffmpeg.mp4","fromWorkId":29545092}],"arguments":[{"name":"prompt","value":""},{"name":"biz","value":"klingai"},{"name":"__initialType","value":"m2v_img2video"},{"name":"__initialPrompt","value":"母亲对着镜头挥手"}]}
# 自定义创意延长
# {"type":"m2v_extend_video","inputs":[{"name":"input","inputType":"URL","url":"https://h2.inkwai.com/bs2/upload-ylab-stunt/special-effect/output/HB1_PROD_ai_web_29542959/396308539942414182/output_ffmpeg.mp4","fromWorkId":29542959}],"arguments":[{"name":"prompt","value":"加点字"},{"name":"biz","value":"klingai"},{"name":"__initialType","value":"m2v_txt2video"},{"name":"__initialPrompt","value":"让佛祖说话，嘴巴要动，像真人一样"}]}
@retrying(max_retries=8, max=8, predicate=lambda r: not r)
async def create_task(request: KlingaiVideoRequest, cookie: Optional[str] = None, vip: bool = False):
    cookie = cookie or await get_next_token_for_polling(FEISHU_URL_VIP if vip else FEISHU_URL, check_token=check_token,
                                                        from_redis=True)

    task_type = TaskType.kling_vip if vip else TaskType.kling

    headers = {
        'Cookie': cookie,
        'Content-Type': 'application/json;charset=UTF-8'
    }
    request_kwargs = {}
    for i in range(3):
        async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, **request_kwargs) as client:
            response = await client.post("/api/task/submit", json=request.payload)
            if response.is_success:
                data = response.json()  # metadata
                send_message(bjson(data))

                # 触发重试 404 429 520
                if any(i in str(data) for i in {"页面未找到", "请求超限", "配额耗尽", "积分消费失败"}):
                    send_message(f"{data}\n\n{cookie}")

                    # 走代理
                    request_kwargs = {
                        "proxy": await get_one_proxy(from_redis=True),
                        # "proxies": proxies,
                    }

                    return

                try:
                    task_ids = jsonpath.jsonpath(data, "$..task.id")  # $..task..[id,arguments]
                    if task_ids:
                        task_id = f"{task_type}-{task_ids[0]}"
                        return Task(id=task_id, data=data, system_fingerprint=cookie)
                    else:
                        return Task(status=0, data=data)

                except Exception as e:
                    logger.error(e)
                    send_message(f"未知错误：{e}")


@retrying(max_retries=3, predicate=lambda r: not r)
async def create_task_plus(request: KlingaiVideoRequest, token: Optional[str] = None, vip: bool = False):
    token = token or await get_next_token_for_polling(FEISHU_URL_VIP if vip else FEISHU_URL, check_token=check_token,
                                                      from_redis=True)

    if not vip: request.mode = "mini"  # 针对普号

    headers = {
        'Cookie': token,
        'Content-Type': 'application/json;charset=UTF-8'
    }

    request_kwargs = {}
    for i in range(3):
        async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100, **request_kwargs) as client:
            response = await client.post("/api/task/submit", json=request.payload)
            if response.is_success:
                data = response.json()
                kling_types.send_message(data)

                # 触发重试 404 429 520
                if any(i in str(data) for i in {"页面未找到", "请求超限", "配额耗尽", "积分消费失败"}):
                    kling_types.send_message(f"{data}\n\n{token}")

                    # 走代理
                    request_kwargs = {
                        "proxy": await get_one_proxy(from_redis=True),
                        # "proxies": proxies,
                    }

                    return

                task_id = message = task_status_msg = ''
                code = data.get("status")
                if data.get("data"):
                    data = data.get("data")
                    if task := data.get("task"):
                        code = 0
                        task_id = f"""kling-video-{request.mode}-{task.get("id")}"""
                        task_status = "processing"
                        task_status_msg = data.get("message")
                    else:
                        task_status = "failed"
                        task_status_msg = data.get("message")

                else:
                    message = data.get("message")
                    task_status = "failed"

                return kling_types.TaskResponse(
                    code=code,
                    message=message,
                    data=kling_types.Task(task_id=task_id, task_status=task_status, task_status_msg=task_status_msg),
                    system_fingerprint=token,
                )


if __name__ == '__main__':
    # https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=v8vcZY

    # cookie = "_did=web_474598569404BC09;did=web_43cec8fac6fa32077f3b12bf131b77a7310f;kuaishou.ai.portal_ph=0e15bfbb51aa3d52d065602d7e566d622ba4;kuaishou.ai.portal_st=ChVrdWFpc2hvdS5haS5wb3J0YWwuc3QSsAHW9jii6pSdK5oTimSzQslDxQ5mLAW8m2j8dKFP4uptOr8aycVOX72ydltRPJhO6QbD4fGYD2pFXD_c4gqsAZPLZluo4DeIxFAWhEplxrmSA61cb-VBtCEB2bxyM1gC-sooTaSESNkekMI5WBEq1_NL5E2x-wigAGi6jm1Chpq-bJ3oTAXe9yMLV3oY4qilfoV9M77nn3cY-r76Z2Z9G-3JICGNjTmN2OAQSf_q3EXtMRoSITGi7VYyBd022GVFnVcqtiPoIiDjv6HmF2lhkn6FgvYhjgLMGinYaZxZj84kbjL8GfxuiygFMAE;userId=415950813"
    # token = "weblogger_did=web_9665177505014D12; _gcl_au=1.1.961392860.1725442018; did=web_d77db05423a7f37445a28355351bf2736bf0; anonymous-message-release-notice-1=true; userId=5138238; ksi18n.ai.portal_st=ChNrc2kxOG4uYWkucG9ydGFsLnN0EqABA02dJVaF0Q5XvWFZVNWLa-7fNeGjTJZ_BP3zPN4JN1Ct8LRrLlT8ZAtqXZa0X_fOBXuQFiOqfoIuTziI8bNUaIKPNjhfcCT2CUcLv9bIVdYkwlJGlzMai8-2o2MoF9yopDzeYfwphWxBh575DrMjMjnPoNp0MqSq3iDIIA9glYGxEOYRcotovl63JPPI1Vy4mSX46S1Omh1X0suF2oWKCBoSPFqCkp3UokbKTh0UPCIW9AgsIiBZRgCwD_UpSF1elCUq5jwFhe-Wr2hErFmS73Zc4GyXOSgFMAE; ksi18n.ai.portal_ph=c52b0d2188bc0e0d209a401f9cb4c4889227"
    request = KlingaiVideoRequest(prompt="一只可爱的黑白边境牧羊犬，头伸出车窗，毛发被风吹动，微笑着伸出舌头",
                                  duration=5)  # 27638649
    # e = KlingaiVideoRequest.Config.json_schema_extra.get('examples')[-1]  # 尾帧
    # request = KlingaiVideoRequest(**e)
    #
    print(arun(create_task(request, vip=True)))
    # arun(get_task("31298981", cookie))
    # arun(get_task("vip-47426519", token))

    # pprint(arun(create_image(rquest)))

    # request
    # request = KlingaiVideoRequest(
    #     prompt="一条可爱的小狗",
    #     url="https://p2.a.kwimgs.com/bs2/upload-ylab-stunt/special-effect/output/HB1_PROD_ai_web_30135907/1706269798026373672/output_ffmpeg.mp4"
    # )
    # pprint(arun(create_task(request, cookie)))
    # pprint(arun(get_task(28106800, cookie)))  # 拓展的id 28106800  可能依赖账号 跨账号失败: 单账号测试成功

    # url = "http://p2.a.kwimgs.com/bs2/upload-ylab-stunt/ai_portal/1720681052/LZcEugmjm4/whqrbrlhpjcfofjfywqqp9.png"
    # request = KlingaiVideoRequest(prompt="狗狗跳起来", url=url)  # 28110824
    # pprint(arun(create_task(request, cookie, feishu_url="https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=EXxwtQ")))

    # pprint(arun(get_task(28110824, cookie)))

    # url = "http://p2.a.kwimgs.com/bs2/upload-ylab-stunt/ai_portal/1720681052/LZcEugmjm4/whqrbrlhpjcfofjfywqqp9.png"

    # request = KlingaiVideoRequest(prompt="狗狗跳起来", url=url)  # 28110824
    # pprint(arun(create_task(request, vip=True)))

    # pprint(arun(get_task(30974235, cookie)))
    # pprint(arun(get_task(28377631, cookie)))
    # pprint(arun(get_task(28383134, cookie)))

    # pprint(arun(beautify_prompt()))

    # 国际
    # cookie = "did=web_b11919c67a1966b83eaef4a19fb2de266cba;ksi18n.ai.portal_ph=644033a151612d07cbdedc21513f5d2191b6;ksi18n.ai.portal_st=ChNrc2kxOG4uYWkucG9ydGFsLnN0EqABChr9Is3s_8NQtSRM3A10k93e-Yg2PRAw5SR8BswQAdCW33dIk_7cWf5EQohyx45HoVG4nYAdpPZRg02_y4vsA5AA2TCzde2-cxAMecVk_Rg_oOnQBGWqMjVachSC82Qf4xA-vpOj1KYRGv4XwQ6ZvpqLRysjpt0543UjaasSUa8sHlD6XT_nwPnOc2LGaIDdhXExgvh85OqK7-FpcvniWRoSSiD-9Kd-wI-i_qkoWz9SxkEvIiDCLGRyg7lgXBHZcPxIy0hnLZfkuyO9AMaVYFzUapkQmCgFMAE;userId=3412057;weblogger_did=web_8897873009A74F8"
    # arun(create_task(request, cookie))

    # cookie = arun(get_next_token_for_polling(feishu_url=FEISHU_URL_VIP, check_token=check_token))
    # arun(get_task("kling@vip-41009887", cookie))

    # with timer():
    #     arun(check_token(cookie))
    #
    #
    #
    #
    # arun(get_reward(token))  # 签到
    # arun(get_point(token))  # 签到

    # arun(get_task_plus('60909700', token=token))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ocr
# @Time         : 2025/10/22 20:57
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


"""
curl https://ai.gitee.com/v1/async/images/ocr \
	-X POST \
	-H "Authorization: Bearer XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" \
	-F "image=@path/to/image.webp" \
	-F "model=DeepSeek-OCR" \
	-F "prompt=<image>\n<|grounding|>Convert the document to markdown." \
	-F "model_size=Gundam"
"""

from meutils.pipe import *
from meutils.llm.clients import AsyncClient
from meutils.schemas.openai_types import CompletionRequest
from meutils.io.files_utils import to_file, to_bytes

from meutils.db.redis_db import redis_aclient
from meutils.caches import rcache

from meutils.apis.utils import make_request_httpx

BASE_URL = "https://ai.gitee.com/v1"


async def get_task(task_id):
    if api_key := await redis_aclient.get(task_id):
        api_key = api_key.decode()

        headers = {
            "Authorization": f"Bearer {api_key}",
        }
        response = await make_request_httpx(
            base_url=BASE_URL,
            path=f"/task/{task_id}",
            headers=headers,
        )
        """
        {'completed_at': 1753692159067,
 'created_at': 1753692151094,
 'output': {'file_url': 'https://gitee-ai.su.bcebos.com/serverless-api/2025-07-28/GB84DX8LK6NUJ0WHZLUNRCXDBFKMVVFH.glb?authorization=bce-auth-v1%2FALTAKZc1TWR1oEpkHMlwBs5YXU%2F2025-07-28T08%3A42%3A38Z%2F604800%2F%2F2799bb11463d736d0eb0fd656c944e02012f7359ca22e8397811f59d306ec353'},
 'started_at': 1753692151358,
 'status': 'success',
 'task_id': 'GB84DX8LK6NUJ0WHZLUNRCXDBFKMVVFH',
 'urls': {'cancel': 'https://ai.gitee.com/api/v1/task/GB84DX8LK6NUJ0WHZLUNRCXDBFKMVVFH/cancel',
          'get': 'https://ai.gitee.com/api/v1/task/GB84DX8LK6NUJ0WHZLUNRCXDBFKMVVFH'}}
        """
        if output := response.get("output"):
            return output


@rcache(ttl=3600)
async def create_task(image, data: Optional[dict] = None, api_key: Optional[str] = None):
    image = await to_bytes(image)

    headers = {
        "Authorization": f"Bearer {api_key}",
    }

    # (filename, file_bytes, mime_type) = image

    response = await make_request_httpx(
        base_url=BASE_URL,
        path="/async/images/ocr",
        headers=headers,
        files={
            "image": image,
        },
        data=data,
    )
    """
    {'created_at': 1753691466227,
 'status': 'waiting',
 'task_id': 'AXA5X0GFGXS96T4ALPYHCTCHAAYANL4U',
 'urls': {'cancel': 'https://ai.gitee.com/api/v1/task/AXA5X0GFGXS96T4ALPYHCTCHAAYANL4U/cancel',
          'get': 'https://ai.gitee.com/api/v1/task/AXA5X0GFGXS96T4ALPYHCTCHAAYANL4U'}}

{'task_id': 'GB84DX8LK6NUJ0WHZLUNRCXDBFKMVVFH'}

    """
    logger.debug(response)
    if task_id := response.get("task_id"):
        await redis_aclient.set(task_id, api_key, ex=24 * 3600)
        return task_id


# {'task_id': 'DBMIQSBXWKNCKNJLDPHEYRML4QNBRXIA'}
class Completions(object):

    def __init__(
            self,
            base_url: Optional[str] = None,
            api_key: Optional[str] = None,
    ):
        self.api_key = api_key

    async def create(self, request: CompletionRequest):
        prompt = """
        <image>\n<|grounding|>Convert the document to markdown.
        <image>\nParse the figure.
        <image>\nLocate <|ref|> the teacher<|/ref|> in the image.
        <image>\nFree OCR.
        """
        system_instruction = None
        if request.system_instruction and request.system_instruction in prompt:
            system_instruction = request.system_instruction

        if not (image_urls := request.last_urls.get("image_url")):
            yield "# Image is required"
            return
        image = image_urls[0]

        data = {
            "model": request.model,  # "DeepSeek-OCR"
            "prompt": system_instruction or "<image>\nFree OCR.",
            "model_size": "Gundam" if not hasattr(request, "model_size") else request.model_size,
        }
        if task_id := await create_task(image=image, data=data, api_key=self.api_key):
            if request.stream:
                yield f"`Task ID: {task_id}`\n\n"

            for i in range(30):
                await asyncio.sleep(3)
                if output := await get_task(task_id=task_id):
                    logger.debug(output)

                    if not request.stream:
                        yield json.dumps(output)
                        return

                    text_result = output['text_result']
                    yield f"""{text_result}\n\n"""

                    if result_image := output.get('result_image'):
                        yield f"""![result_image]({result_image})"""

                    return
                else:
                    if request.stream:
                        yield "### 处理中\n"


if __name__ == '__main__':
    """
    <image>\n<|grounding|>Convert the document to markdown.
    <image>\nParse the figure.
    <image>\nLocate <|ref|> the teacher<|/ref|> in the image.
    <image>\nFree OCR.
    """
    api_key = "NWVXUPI38OQVXZGOEL3D23I9YUQWZPV23GVVBW1X"
    data = {
        "model": "DeepSeek-OCR",
        "prompt": "<image>\n<|grounding|>Convert the document to markdown.",
        "model_size": "Gundam",
    }

    image = "https://s3.ffire.cc/files/pdf_to_markdown.jpg"

    # arun(create_task(image=image, data=data, api_key=api_key))

    arun(get_task(task_id='DBMIQSBXWKNCKNJLDPHEYRML4QNBRXIA'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : screenshot
# @Time         : 2024/9/18 16:02
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.io.files_utils import to_url
from meutils.playwright_utils import get_new_page
from meutils.schemas.playwright_types import ScreenshotRequest

from fastapi import APIRouter, File, UploadFile, Query, Form, Response, Request

router = APIRouter()

HTML_PARSER = re.compile(r'```html(.*?)```', re.DOTALL)


@router.post("/screenshot")  # todo: 多个url
async def capture_screenshot(
        request: ScreenshotRequest,
):
    htmls = HTML_PARSER.findall(request.html)
    if htmls:
        request.html = htmls[0]

    page = await get_new_page()
    await page.set_content(request.html)
    await page.set_viewport_size({"width": request.width, "height": request.height})

    with tempfile.NamedTemporaryFile(suffix='.png') as file:
        await page.screenshot(path=file.name)
        url = await to_url(file.name)
        page.close()
        return {"url": url}


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()
    app.include_router(router)
    app.run(port=39666)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : web_search
# @Time         : 2024/5/10 11:17
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import langchain
from meutils.pipe import *
import urllib, json

def results(original_query):
    search_specs = [
         ["Google", "~googlequery", "https://www.google.com/search?q="],
         ["Duck Duck Go", "~duckduckgoquery", "https://duckduckgo.com/?q="],
         ["Google Images", "~googleimagequery", "https://www.google.com/search?tbm=isch&q="],
         ["Baidu", "~baiduquery", "http://www.baidu.com/s?wd="],
         ["Bing", "~bingquery", "http://www.bing.com/search?q="],
         ["Yahoo", "~yahooquery", "https://sg.search.yahoo.com/search?p="],
         ["Twitter", "~twitterquery", "https://mobile.twitter.com/search?q="],
         ["Reddit", "~redditquery", "https://www.reddit.com/search?q="]
    ]





if __name__ == '__main__':
    results(original_query="周杰伦")


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : fileparser
# @Time         : 2025/1/7 17:48
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://bigmodel.cn/dev/activities/freebie/fileextracion
import asyncio

from meutils.pipe import *
from meutils.io.files_utils import to_bytes, guess_mime_type
from meutils.llm.clients import moonshot_client, zhipuai_client, APIStatusError
from meutils.notice.feishu import send_message as _send_message, FILES
from meutils.caches.redis_cache import cache

send_message = partial(
    _send_message,
    title=__name__,
    url=FILES
)
"""

# 智谱
# 格式限制：.PDF .DOCX .DOC .XLS .XLSX .PPT .PPTX .PNG .JPG .JPEG .CSV .PY .TXT .MD .BMP .GIF

# kimi todo: 定期删除文件
文件接口与 Kimi 智能助手中上传文件功能所使用的相同，支持相同的文件格式，它们包括 
.pdf .txt .csv .doc .docx .xls .xlsx .ppt .pptx .md .jpeg .png .bmp .gif .svg .svgz .webp .ico .xbm .dib .pjp .tif 
.pjpeg .avif .dot .apng .epub .tiff .jfif .html .json .mobi .log .go .h .c .cpp .cxx .cc .cs .java .js .css .jsp .php 
.py .py3 .asp .yaml .yml .ini .conf .ts .tsx 等格式。

# todo: 
+ .sh
"""


@cache(ttl=24 * 3600)
async def file_extract(
        file,
        provider: Union[str, Literal['kimi', 'moonshot', 'zhipu']] = 'moonshot'
):
    """todo 定时删除文件
    todo: 兼容下 url bytes path

    # moonshot_client.files.

# len(moonshot_client.files.list().data)

    """
    mime_type = guess_mime_type(file)
    if mime_type == "application/octet-stream": return  # 不解析

    logger.debug(f"file_extract: {mime_type}")

    if provider == "zhipu":
        client = zhipuai_client
    else:
        client = moonshot_client  # 默认

    filename = Path(file).name
    file_bytes: bytes = await to_bytes(file)

    # todo: zhipu兜底
    try:
        file_object = await client.files.create(
            # file=file,
            # file=("filename.pdf", file),
            file=(filename, file_bytes, mime_type),
            purpose="file-extract"
        )

        if isinstance(file, str) and file.startswith('http'):
            file_object.url = file
        send_message(file_object)

        response = await client.files.content(file_id=file_object.id)  # 抛错处理
        return response.json()
    except APIStatusError as e:
        logger.debug(e)
        _ = e.response.json()
        if isinstance(file, str) and file.startswith('http'):
            _['url'] = file
        send_message(_)
        return _


if __name__ == '__main__':
    # file = "https://oss.ffire.cc/files/招标文件备案表（第二次）.pdf"
    file = "https://oss.ffire.cc/files/%E6%8B%9B%E6%A0%87%E6%96%87%E4%BB%B6%E5%A4%87%E6%A1%88%E8%A1%A8%EF%BC%88%E7%AC%AC%E4%BA%8C%E6%AC%A1%EF%BC%89.pdf"
    "https://oss.ffire.cc/files/%E6%8B%9B%E6%A0%87%E6%96%87%E4%BB%B6%E5%A4%87%E6%A1%88%E8%A1%A8%EF%BC%88%E7%AC%AC%E4%BA%8C%E6%AC%A1%EF%BC%89.pdf 这个文件讲了什么？"
    # file = "https://oss.ffire.cc/files/百炼系列手机产品介绍.docx"
    # file = Path("/Users/betterme/PycharmProjects/AI/MeUtils/meutils/llm/completions/rag/百炼系列手机产品介绍.docx")

    # file = "/Users/betterme/PycharmProjects/AI/MeUtils/meutils/io/img_1.png"

    # openai.BadRequestError: Error code: 400 - {'error': {'message': 'text extract error: 没有解析出内容', 'type': 'invalid_request_error'}}
    # file = "https://oss.ffire.cc/files/kling_watermark.png"

    # with timer():
    #     r = arun(file_extract(file, moonshot_client))

    # with timer():
    #     r = arun(file_extract(file, provider='kimi'))

    # with timer():
    #     arun(file_extract(file))

    print(guess_mime_type('x.sh'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : demo
# @Time         : 2024/9/18 10:39
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import re

from meutils.pipe import *

HTML_PARSER = re.compile(r'```html(.*?)```', re.DOTALL)
s = """
这是一堆文本
```html
这是一段html
```
这是一堆文本
"""

print(HTML_PARSER.findall(s))

# {"messages":[{"role":"user","content":"早饭"}],"chat_id":"ssZZTRs","model":"Qwen/Qwen2-Math-72B-Instruct"}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : request_types
# @Time         : 2024/12/20 16:29
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

# requests.request()


class RequestTypes(BaseModel):
    method: str = "POST"

    base_url: str
    api_key: Optional[str] = None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : watermark_remove
# @Time         : 2025/3/23 11:18
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
import requests
import json


def get_file_content(filePath):
    with open(filePath, 'rb') as fp:
        return fp.read()


class CommonOcr(object):
    def __init__(self, img_path=None, is_url=False):
        # 图像水印去除
        self._url = 'https://api.textin.com/ai/service/v1/image/watermark_remove'
        # 请登录后前往 “工作台-账号设置-开发者信息” 查看 x-ti-app-id
        # 示例代码中 x-ti-app-id 非真实数据
        self._app_id = 'c81f*************************e9ff'
        # 请登录后前往 “工作台-账号设置-开发者信息” 查看 x-ti-secret-code
        # 示例代码中 x-ti-secret-code 非真实数据
        self._secret_code = '5508***********************1c17'
        self._img_path = img_path
        self._is_url = is_url

    def recognize(self):
        head = {}
        try:
            head['x-ti-app-id'] = self._app_id
            head['x-ti-secret-code'] = self._secret_code
            if self._is_url:
                head['Content-Type'] = 'text/plain'
                body = self._img_path
            else:
                image = get_file_content(self._img_path)
                head['Content-Type'] = 'application/octet-stream'
                body = image
            result = requests.post(self._url, data=body, headers=head)
            return result.text
        except Exception as e:
            return e


if __name__ == "__main__":
    # 示例 1：传输文件
    response = CommonOcr(img_path=r'example.jpg')
    print(response.recognize())
    # 示例 2：传输 URL
    response = CommonOcr(img_path='http://example.com/example.jpg', is_url=True)
    print(response.recognize())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : path_utils
# @Time         : 2020/11/12 11:39 上午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : 

import os
import sys
import json
import zipfile

import yaml

from pathlib import Path
from zipfile import ZipFile
from loguru import logger

get_resolve_path = lambda path, file: (Path(file).parent / Path(path)).resolve()
get_module_path = lambda path, file: os.path.normpath(os.path.join(os.getcwd(), os.path.dirname(file), path))


def file2json(path):
    """yaml/json"""
    p = Path(path)

    o = {}
    if p.is_file():

        if p.name.endswith('.json'):
            o = json.loads(p.read_bytes())

        elif p.name.endswith(('.yml', '.yaml')):
            o = yaml.safe_load(p.read_bytes())
    else:
        logger.error(f"无效地址：{path}")
    return o


def sys_path_append(path, __file__=None):
    """添加home到系统
    """
    if __file__:
        path = get_module_path(path, __file__)

    p = Path(path).resolve()
    if p.is_file():
        py_home = p.parent.__str__()
    elif p.is_dir():
        py_home = p.__str__()
    else:
        raise Exception(f"{path} 为错误路径")

    logger.info(f"{path} HOME: {py_home}")

    sys.path.append(py_home)

    return py_home


def path2list(path, pattern='*'):
    ps = []
    p = Path(path)
    if p.is_file():
        ps = [p]
    elif p.is_dir():
        ps = list(p.glob(pattern))
    else:
        logger.error('ErrorPath !!!')
    return ps


def get_config(config_init=None):
    if isinstance(config_init, str) and Path(config_init).is_file():
        p = Path(config_init)

        if p.name.endswith('.json'):
            config_init = json.loads(p.read_bytes())

        elif p.name.endswith('.yaml') or p.name.endswith('.yml'):
            config_init = yaml.safe_load(p.read_bytes())

        else:
            logger.error('目前只支持dict/json/yaml格式')
            config_init = {}

    else:
        config_init = config_init if config_init else {}

    return config_init


def get_import_module_path(name='paddleocr'):
    import importlib
    return str(importlib.import_module(name)).split()[-1][1:-2]


def zipfiles(files, out_file='file.zip', compression=0):
    """
    0 ZIP_STORED：不进行压缩，直接存储
    8 ZIP_DEFLATED：使用DEFLATE算法进行压缩，压缩率较高
    12 ZIP_BZIP2：使用BZIP2算法进行压缩，在某些情况下比DEFLATE更优秀
    14 ZIP_LZMA：使用LZMA算法进行压缩，压缩率最高
    """
    with ZipFile(out_file, compression=compression, mode='w') as f:
        for file in files:
            f.write(file)

    return out_file


if __name__ == '__main__':
    print(os.getcwd())
    print(os.path.dirname(__file__))



"""Gunicorn configuration file.

List of all available settings:
https://docs.gunicorn.org/en/latest/settings.html

"""
import os

# Server socket
#
#   bind - The socket to bind.
#
#       A string of the form: 'HOST', 'HOST:PORT', 'unix:PATH'.
#       An IP is a valid HOST.
#
#   backlog - The number of pending connections. This refers
#       to the number of clients that can be waiting to be
#       served. Exceeding this number results in the client
#       getting an error when attempting to connect. It should
#       only affect servers under significant load.
#
#       Must be a positive integer. Generally set in the 64-2048
#       range.
#

bind = "{host}:{port}".format(
    host=os.getenv("FASTAPI_HOST", "0.0.0.0"),
    port=os.getenv("FASTAPI_PORT", "8000"),
)
backlog = 1024

#
# Worker processes
#
#   workers - The number of worker processes that this server
#       should keep alive for handling requests.
#
#       A positive integer generally in the 2-4 x $(NUM_CORES)
#       range. You'll want to vary this a bit to find the best
#       for your particular application's work load.
#
#   worker_class - The type of workers to use. The default
#       sync class should handle most 'normal' types of work
#       loads. You'll want to read
#       http://docs.gunicorn.org/en/latest/design.html#choosing-a-worker-type
#       for information on when you might want to choose one
#       of the other worker classes.
#
#       A string referring to a Python path to a subclass of
#       gunicorn.workers.base.Worker. The default provided values
#       can be seen at
#       http://docs.gunicorn.org/en/latest/settings.html#worker-class
#
#   worker_connections - For the eventlet and gevent worker classes
#       this limits the maximum number of simultaneous clients that
#       a single process can handle.
#
#       A positive integer generally set to around 1000.
#
#   timeout - If a worker does not notify the master process in this
#       number of seconds it is killed and a new worker is spawned
#       to replace it.
#
#       Generally set to thirty seconds. Only set this noticeably
#       higher if you're sure of the repercussions for sync workers.
#       For the non sync workers it just means that the worker
#       process is still communicating and is not tied to the length
#       of time required to handle a single request.
#
#   keepalive - The number of seconds to wait for the next request
#       on a Keep-Alive HTTP connection.
#
#       A positive integer. Generally set in the 1-5 seconds range.
#
#   reload - Restart workers when code changes.
#
#       True or False

workers = int(os.getenv("FASTAPI_WORKERS", 2))
worker_class = "uvicorn.workers.UvicornWorker"
worker_connections = 1024
timeout = 360
graceful_timeout = 128  # 在接收到重新启动信号后，worker有这么多时间来完成服务请求。超时后（从接收到重启信号开始）仍然活着的worder被强制杀死。
keepalive = 16
reload = False

#
#   spew - Install a trace function that spews every line of Python
#       that is executed when running the server. This is the
#       nuclear option.
#
#       True or False
#

spew = False

#
# Server mechanics
#
#   daemon - Detach the main Gunicorn process from the controlling
#       terminal with a standard fork/fork sequence.
#
#       True or False
#
#   raw_env - Pass environment variables to the execution environment.
#
#   pidfile - The path to a pid file to write
#
#       A path string or None to not write a pid file.
#
#   user - Switch worker processes to run as this user.
#
#       A valid user id (as an integer) or the name of a user that
#       can be retrieved with a call to pwd.getpwnam(value) or None
#       to not change the worker process user.
#
#   group - Switch worker process to run as this group.
#
#       A valid group id (as an integer) or the name of a user that
#       can be retrieved with a call to pwd.getgrnam(value) or None
#       to change the worker processes group.
#
#   umask - A mask for file permissions written by Gunicorn. Note that
#       this affects unix socket permissions.
#
#       A valid value for the os.umask(mode) call or a string
#       compatible with int(value, 0) (0 means Python guesses
#       the base, so values like "0", "0xFF", "0022" are valid
#       for decimal, hex, and octal representations)
#
#   tmp_upload_dir - A directory to store temporary request data when
#       requests are read. This will most likely be disappearing soon.
#
#       A path to a directory where the process owner can write. Or
#       None to signal that Python should choose one on its own.
#

daemon = False
# raw_env = [
#     'DJANGO_SECRET_KEY=something',
#     'SPAM=eggs',
# ]
umask = 0
user = None
group = None
tmp_upload_dir = None

#
#   Logging
#
#   logfile - The path to a log file to write to.
#
#       A path string. "-" means log to stdout.
#
#   loglevel - The granularity of log output
#
#       A string of "debug", "info", "warning", "error", "critical"
#

pidfile = None
proc_name = None

errorlog = 'errorlog.log'
accesslog = 'accesslog.log'

# errorlog = "-"
# accesslog = "-"

loglevel = os.getenv("FASTAPI_GUNICORN_LOG_LEVEL", "info")
access_log_format = os.getenv(
    "FASTAPI_GUNICORN_LOG_FORMAT",
    '%(h)s %(l)s %(u)s %(t)s "%(r)s" %(s)s %(b)s "%(f)s" "%(a)s"',
)

logger_class = "meutils.logging.logger.GunicornLogger"


#
# Process naming
#
#   proc_name - A base to use with setproctitle to change the way
#       that Gunicorn processes are reported in the system process
#       table. This affects things like 'ps' and 'top'. If you're
#       going to be running more than one instance of Gunicorn you'll
#       probably want to set a name to tell them apart. This requires
#       that you install the setproctitle module.
#
#       A string or None to choose a default of something like 'gunicorn'.
#


#
# Server hooks
#
#   post_fork - Called just after a worker has been forked.
#
#       A callable that takes a server and worker instance
#       as arguments.
#
#   pre_fork - Called just prior to forking the worker subprocess.
#
#       A callable that accepts the same arguments as after_fork
#
#   pre_exec - Called just prior to forking off a secondary
#       master process during things like config reloading.
#
#       A callable that takes a server instance as the sole argument.
#


def post_fork(server, worker):
    """Execute after a worker is forked."""
    server.log.info("Worker spawned (pid: %s)", worker.pid)


def pre_fork(server, worker):
    """Execute before a worker is forked."""
    pass


def pre_exec(server):
    """Execute before a new master process is forked."""
    server.log.info("Forked child, re-executing.")


def when_ready(server):
    """Execute just after the server is started."""
    server.log.info("Server is ready. Spawning workers")


def worker_int(worker):
    """Execute just after a worker exited on SIGINT or SIGQUIT."""
    worker.log.info("worker received INT or QUIT signal")

    # get traceback info
    import threading
    import sys
    import traceback

    id2name = {th.ident: th.name for th in threading.enumerate()}
    code = []

    for threadId, stack in sys._current_frames().items():
        code.append(
            "\n# Thread: %s(%d)" % (id2name.get(threadId, ""), threadId)
        )
        for filename, lineno, name, line in traceback.extract_stack(stack):
            code.append('File: "%s", line %d, in %s' % (filename, lineno, name))
            if line:
                code.append("  %s" % (line.strip()))

    worker.log.debug("\n".join(code))


def worker_abort(worker):
    """Execute when worker received the SIGABRT signal."""
    worker.log.info("worker received SIGABRT signal")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : wechat
# @Time         : 2021/6/7 11:17 上午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  :
import time

from meutils.pipe import *
from meutils.notice.wechat import Bot

bot = Bot("5f51e925-1257-4f90-88ea-f474bc1e2a05")

json = {
    "msgtype": "news",
    "news": {
        "articles": [
            {
                "title": "数据科学研发中心周会",  # 14个字换行
                "description": "同学们，点击图片进教室，迟到发红包",  # 17个字换行
                "url": "https://meeting.tencent.com/s/UJ5c5nOdCJI3",
                "picurl": "https://i.loli.net/2021/08/03/CJeHpGuWXKrPgc3.jpg",  # 1068*455 150*150
            },
        ]
    }
}
logger.info("周会通知")
bot.send(json)

time.sleep(60 * 15)
json = {
    "msgtype": "text",
    "text": {
        "mentioned_mobile_list": ["@all"],
        "content": "点击图片进教室，周会已经开始啦",
    }
}
logger.info("周会开始")
bot.send(json)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2025/10/13 13:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from meutils.db.redis_db import redis_aclient
from meutils.llm.clients import AsyncClient
from meutils.notice.feishu import send_message_for_images
from meutils.io.files_utils import to_url, to_url_fal
from meutils.schemas.video_types import SoraVideoRequest, Video

from openai import APIStatusError

from meutils.apis.translator import deeplx

MODELS_MAPPING = {
    "sora2": "openai:3@1",
}


async def create_task(request: SoraVideoRequest, api_key: str, base_url: Optional[str] = None):
    payload = [
        {
            "taskType": "videoInference",

            "taskUUID": str(uuid.uuid4()),

            "model": MODELS_MAPPING.get(request.model, request.model),
            "positivePrompt": request.prompt,

            "numberResults": 1,

            "duration": int(request.seconds),

            "includeCost": True,

            "webhookURL": "https://openai-dev.chatfire.cn/sys/webhook/runware?expr=%24..taskUUID"  # task_id
        },

    ]

    payload[0]["width"], payload[0]["height"] = map(int, request.size.split("x"))

    if request.input_reference:  # todo 多图
        if not isinstance(request.input_reference, list):
            request.input_reference = [request.input_reference]

        urls = await to_url_fal(request.input_reference, content_type="image/png")
        payload[0]["frameImages"] = [
            {
                "inputImage": url
            }
            for url in urls
        ]

    logger.debug(bjson(payload))

    try:
        client = AsyncClient(base_url="https://api.runware.ai/v1", api_key=api_key, timeout=300)
        response = await client.post(
            "/",
            body=payload,
            cast_to=object
        )

        video = Video(
            id=response["data"][0]["taskUUID"],
            model=request.model,
            seconds=request.seconds,
        )

        # await redis_aclient.set(video.id, api_key) # 回调接口即可

        return video

    except APIStatusError as e:
        if (errors := e.response.json().get("errors")):
            logger.debug(bjson(errors))

        raise e


async def get_task(task_id):
    video = Video(id=task_id)

    webhook_id = f"webhook:runware:{task_id}"
    if not (data := await redis_aclient.lrange(webhook_id, 0, -1)):
        raise ValueError(f"task_id not found")

    runware_response = json.loads(data[0])
    if data := runware_response["data"]:
        if video_url := data[0].get("videoURL"):
            video.progress = 100
            video.video_url = video_url

    return video


if __name__ == '__main__':
    model = "openai:3@1"

    request = SoraVideoRequest(model=model, prompt="a cat")

    # logger.info(request)

    # arun(generate(request, api_key="Fk3Clsgcwc3faIvbsjDajGFATJLfaWpE"))

    # https://openai-dev.chatfire.cn/sys/webhook/runware?expr=%24..taskUUID
    arun(get_task("3f7fcfc4-e257-4cb7-bf97-439bceaa7cc3"))

    # {
    #     "data": [
    #         {
    #             "taskType": "videoInference",
    #             "taskUUID": "8a5a1c09-d0a5-4b1b-9b67-8943cacc935f"
    #         }
    #     ]
    # }

"""
curl --request POST \
--url 'https://api.runware.ai/v1' \
--header "Authorization: Bearer Fk3Clsgcwc3faIvbsjDajGFATJLfaWpE" \
--header "Content-Type: application/json" \
--data-raw '[
  {
    "taskType": "videoInference",
    "duration": 4,
    "fps": 30,
    "height": 720,
    "width": 1280,
    "model": "openai:3@1",
    "numberResults": 1,
    "positivePrompt": "a cat",
    "taskUUID": "3f7fcfc4-e257-4cb7-bf97-439bceaa7cc3",
    "webhookURL": "https://openai-dev.chatfire.cn/sys/webhook/runware?expr=%24..taskUUID"
  }
]'
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ips
# @Time         : 2024/9/27 12:02
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo: 增加代理检测
# https://github.com/Sndeok/HarbourJ/blob/5bbbc4b6118df779929d1521270864753731b78a/jd_inviteDraw.py#L117

from meutils.pipe import *
from meutils.caches import rcache
from meutils.decorators.retry import retrying
from meutils.hash_utils import murmurhash
from meutils.config_utils.lark_utils import get_next_token_for_polling, get_series

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/XfjqszII3hZAEvtTOgecOgv2nye?sheet=c14b34"
FEISHU_URL_METASO = "https://xchatllm.feishu.cn/sheets/XfjqszII3hZAEvtTOgecOgv2nye?sheet=MXvEIN"

BASE_URL = "http://ip.chatfire.cn"
# BASE_URL = "http://api.xiequ.cn"

IPS = "58.240.255.226,110.42.51.201,154.37.214.121,154.40.54.76"


@rcache(ttl=60)
async def get_one_proxy(token: Optional[str] = None, feishu_url: str = "", from_redis: bool = True):
    feishu_url = feishu_url or FEISHU_URL
    if token:  # 按token分配固定的ip
        ips = await get_series(feishu_url=feishu_url, duplicated=True)

        idx = murmurhash(token, bins=len(ips))
        logger.debug(f"{idx}: {ips}")
        ip = ips[idx]

    else:
        # 随机获取ip: 如果从redis就是循环队列 check_proxy
        ip = await get_next_token_for_polling(feishu_url, from_redis=from_redis)
    # https://github.com/encode/httpx/pull/2879
    # proxies = {
    #     "http://": f"http://{ip}",  # 自建 38443
    #     "https://": f"http://{ip}",
    # }

    proxy = f"http://{ip}"

    logger.debug(f"proxy: {proxy}")

    return proxy


def checkip():  # 获取本地ip
    url = 'http://api.xiequ.cn/VAD/OnlyIp.aspx?yyy=123'
    response = requests.get(url).text
    return response


# ukey=7C4B6F4BF696311B9AB961F253184A20
# key=1ea276146ccb4149bf96f1cfcee1973a
async def add_ips(ips: Optional[str] = None):
    ips = (ips or IPS).split(',')
    params = {
        "uid": "134597",
        "ukey": "42EB7F0B846C187F1FDAF28873AE759E",
        "act": "del",
        "ip": "all"
    }
    async with httpx.AsyncClient(base_url=BASE_URL, timeout=100) as client:
        resp = await client.get("/IpWhiteList.aspx", params=params)
        logger.debug(f"删除：{resp.text}")

        for ip in ips:
            params['ip'] = ip
            params['act'] = "add"
            resp = await client.get("/", params=params)

            logger.debug(f"添加：{resp.text}")
            await asyncio.sleep(5)


@rcache(ttl=2.5 * 60)
@retrying()
async def get_proxies(request_type: str = "httpx", **kwargs):
    params = {
        'num': '1',

        'act': 'get',
        'uid': '134597',
        'vkey': '07A1D82FDDE1E96DB5CEF4EF12C8125F',
        'time': '30',
        'plat': '1',
        're': '1',
        'type': '0',
        'so': '1',
        'ow': '1',
        'spl': '1',
        'db': '1'
    }
    async with httpx.AsyncClient(base_url=BASE_URL, timeout=100) as client:
        resp = await client.get("/VAD/GetIp.aspx", params=params)
        ip = resp.text
        if request_type == "httpx":
            return f"http://{ip}"
        else:

            proxies = {
                # "http://": f"socks5://{ip}",
                # "https://": f"socks5://{ip}",

                "http": f"http://{ip}",
                "https": f"http://{ip}",

            }
        logger.debug(proxies)
        return proxies


async def check_proxy(proxy):
    try:
        async with httpx.AsyncClient(proxy=proxy) as client:
            response = await client.get("http://icanhazip.com", timeout=5)  # http://httpbin.org/ip
            logger.debug(response.status_code)

            return response.text
    except Exception as e:
        logger.error(e)
        return False


if __name__ == '__main__':
    proxies = arun(get_proxies())
    # url = "http://api.xiequ.cn/VAD/GetIp.aspx?act=get&uid=134597&vkey=07A1D82FDDE1E96DB5CEF4EF12C8125F&num=1&time=30&plat=1&re=1&type=0&so=1&ow=1&spl=1&addr=&db=1"
    # url = f"{BASE_URL}/VAD/GetIp.aspx?act=get&uid=134597&vkey=07A1D82FDDE1E96DB5CEF4EF12C8125F&num=1&time=30&plat=1&re=1&type=0&so=1&ow=1&spl=1&addr=&db=1"
    #
    # resp = httpx.Client(
    #     proxies=proxies,
    #     timeout=100
    # ).get(url)
    #
    # logger.debug(resp.text)
    ip = "http://proxy.chatfire.cn"
    ip = "http://chatfire:chatfire@110.42.51.201:38443"
    ip = "http://154.9.253.9:38443"
    ip = "http://154.9.252.28:38443"
    ip = "http://110.42.51.223:38443"
    ip = "http://154.40.54.76:38443"
    # proxy = "http://120.26.134.112:22443"

    arun(check_proxy(proxies))

    # arun(get_one_proxy())

    # arun(add_ips())
    # arun(get_one_proxy(from_redis=True))
    # arun(get_one_proxy("FEISHU_URL", exclude_ips="154.40.54.76"))
    # arun(get_one_proxy("FEISHU_URL", exclude_ips="154.40.54.76", from_redis=True))
    # arun(get_one_proxy(from_redis=True))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2025/6/18 16:34
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.db.redis_db import redis_aclient
from meutils.apis.utils import make_request
from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status, BackgroundTasks

base_url = "https://api.gptgod.online"


async def create_task(
        request: dict,
        api_key: Optional[str] = None
):
    response = await make_request(
        base_url=base_url,
        api_key=api_key,
        path="/v1/video/create",
        payload=request,
    )

    if task_id := response.get('id'):
        await redis_aclient.set(task_id, api_key, ex=7 * 24 * 3600)

    return response


async def get_task(task_id: str):
    token = await redis_aclient.get(task_id)  # 绑定对应的 token
    token = token and token.decode()
    if not token:
        raise HTTPException(status_code=404, detail="TaskID not found")

    response = await make_request(
        base_url=base_url,
        api_key=token,
        path=f"/v1/video/query?id={task_id}",
        method="GET",
    )
    return response


if __name__ == '__main__':
    api_key = "sk-h0Cgw9qeyotIUC9WnWSBEB0aO4RbjgEbhZmtg2Ja0kL5npDZ1"
    payload = {
        "prompt": "牛飞上天了",
        "model": "veo3",
        "images": [
            "https://filesystem.site/cdn/20250612/VfgB5ubjInVt8sG6rzMppxnu7gEfde.png",
            "https://filesystem.site/cdn/20250612/998IGmUiM2koBGZM3UnZeImbPBNIUL.png"
        ],
        "enhance_prompt": True
    }

    # arun(create_task(payload, api_key))

    arun(get_task("veo3:2ba161ec-747f-4d5b-b58b-2a610bfc2c31"))






#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : deepinfra
# @Time         : 2024/11/26 13:57
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from openai import AsyncOpenAI

from meutils.pipe import *
from meutils.decorators.retry import retrying
from meutils.io.files_utils import to_bytes
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.notice.feishu import send_message as _send_message, AUDIO
from meutils.schemas.openai_types import STTRequest

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=6lr4oi"

BASE_URL = os.getenv("DEEPINFRA_BASE_URL")

DEFAULT_MODEL = "openai/whisper-large-v3-turbo"
MODELS = {
    "whisper-large-v3": "openai/whisper-large-v3",
    "whisper-large-v3-turbo": "openai/whisper-large-v3-turbo",
    "whisper-1": "distil-whisper/distil-large-v3"
}

send_message = partial(
    _send_message,
    title=__name__,
    url=AUDIO
)


@retrying(max_retries=3, title=__name__)
async def asr(request: STTRequest, api_key: Optional[str] = None):
    api_key = api_key or await get_next_token_for_polling(FEISHU_URL)

    request.model = MODELS.get(request.model, DEFAULT_MODEL)
    request.file = ('_.mp3', await to_bytes(request.file))

    data = request.model_dump(exclude_none=True)

    client = AsyncOpenAI(base_url=BASE_URL, api_key=api_key)
    response = await client.audio.transcriptions.create(**data)
    return response


if __name__ == '__main__':
    request = STTRequest(
        model="whisper-1",
        file=Path("/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_audio/asr/audio.mp3").read_bytes()
    )

    arun(asr(request))
    # print(MODELS.values())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : md
# @Time         : 2024/11/25 15:33
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://readpo.com/en/poster#%201
# https://readpo.com/zh/poster

from meutils.pipe import *
from urllib.parse import quote


@background_task
def get(url):
    resp = httpx.get(url, timeout=30)
    # Path('x.png').write_bytes(resp.content)
    # logger.debug(resp.text)
    logger.debug(resp.status_code)


def markdown2poster(title="MarkdownTest", content="# fire"):
    content = f"""https://readpo.com/p/{quote(content)}"""

    # content = f"""https://md.chatfire.cn/{quote(content)}"""
    get(content)

    return f"![{title}]({content})"

if __name__ == '__main__':
    title = "🔥最新功能"
    content = """
    # 最新通知
> 支持超多模型：`对话` `图片` `视频` `语音` `音乐` `变清晰` `去水印` `文档解析` `联网API`
---

## 2025.02.21
> jina-deepsearch 满血r1 `推理` `搜索`


> Grok-3 是马斯克旗下xAI公司开发的人工智能模型，具备128,000个Token的上下文处理能力，支持函数调用和系统提示，并计划推出多模态版本以处理图像。
  - grok-3
  - grok-3-reasoner `推理`
  - grok-3-deepsearch `搜索`



    """.strip()
    print(markdown2poster(title, content))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-ANN.
# @File         : ann_service
# @Time         : 2021/1/31 11:25 下午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  :

from meutils.zk_utils import *
from appzoo import App
from annzoo.ann import ANN, Collection


# 实时更新配置
@zk.DataWatch('/push/easyann/ann_service')
def watcher(data, stat):  # (data, stat, event)
    ZKConfig.info = yaml.safe_load(data)  # biz2ip



biz2client = {biz: ANN(ip) for biz, ip in ZKConfig.info.items()}


def searcher(**kwargs):
    # DataModel
    biz = kwargs.get('biz')
    collection = kwargs.get('collection')

    topk = kwargs.get('topk', 10)
    nprobe = kwargs.get('nprobe', 1)
    scalar_list = kwargs.get('scalar_list', 1)
    vectors = kwargs.get('vectors', None)

    if vectors is None:
        ids = kwargs.get('ids', None)
        # id mapping
        vectors = ...

    c: Collection = biz2client[biz].__getattr__(collection)
    c.search(vectors, topk, nprobe, scalar_list)


if __name__ == '__main__':
    app = App()
    app.add_route('/', searcher, method='POST')

    app.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : model_info
# @Time         : 2024/11/21 15:53
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.schemas.oneapi import icons

model_info = {
    "ai-search": {
        "group": "Chatfire",
        "icon": "https://registry.npmmirror.com/@lobehub/assets-emoji-anim/1.0.0/files/assets/cowboy-hat-face.webp"
    },
    "gpt-4o": {
        "group": "OpenAI",

        # "name": "Gpt-4O",
        # "note": "gpt-4o",
        "tags": "nb",
    },

    "gpt-4o-all": {
        "group": "OpenAI Plus",

        # "name": "Gpt-4O",
        # "note": "gpt-4o",
        "tags": "nb-all",
    }
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : config
# @Time         : 2024/11/28 14:47
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from datetime import timedelta

from celery import Task, shared_task
from kombu import Exchange, Queue
from tenacity import retry, stop_after_attempt, wait_exponential


class RetryableTask(Task):
    # 使用 tenacity 进行更灵活的重试控制
    @retry(
        stop=stop_after_attempt(5),
        wait=wait_exponential(multiplier=1, min=4, max=10),
        retry_error_callback=lambda retry_state: handle_final_failure(retry_state)
    )
    def run_with_retry(self, *args, **kwargs):
        return self.run(*args, **kwargs)

    def __call__(self, *args, **kwargs):
        return self.run_with_retry(*args, **kwargs)


# 基础配置
broker_url = 'redis://localhost:6379/0'  # 消息代理（推荐使用Redis）
result_backend = 'redis://localhost:6379/1'  # 结果存储

# 时区设置
timezone = 'Asia/Shanghai'
enable_utc = False

# 任务序列化与反序列化配置
task_serializer = 'json'
result_serializer = 'json'
accept_content = ['json']

# 任务执行设置
task_soft_time_limit = 10 * 60  # 任务软超时时间（秒）
task_time_limit = 6 * task_soft_time_limit  # 任务硬超时时间（秒）

# 并发配置
worker_concurrency = 8  # worker并发数，一般设置为CPU核心数
worker_prefetch_multiplier = 4  # 预取任务数
worker_max_tasks_per_child = 256  # 每个worker执行多少个任务后自动重启

# 任务队列配置
task_default_queue = 'default'  # 默认队列
task_queues = (
    Queue('default', Exchange('default'), routing_key='default'),
    Queue('high_priority', Exchange('high_priority'), routing_key='high_priority'),
    Queue('low_priority', Exchange('low_priority'), routing_key='low_priority'),
)

# 任务重试配置
task_publish_retry = True  # 发布任务失败时重试
task_publish_retry_policy = {
    'max_retries': 3,  # 最大重试次数
    'interval_start': 0,  # 初始重试等待时间
    'interval_step': 0.2,  # 重试间隔递增步长
    'interval_max': 0.2,  # 最大重试间隔
}

# # 任务路由配置
# task_routes = {
#     'project.tasks.high_priority_task': {'queue': 'high_priority'},
#     'project.tasks.low_priority_task': {'queue': 'low_priority'},
# }
#
# # 定时任务配置
# beat_schedule = {
#     'task-every-30-seconds': {
#         'task': 'project.tasks.periodic_task',
#         'schedule': timedelta(seconds=30),
#     },
# }

# 日志配置
worker_log_format = '[%(asctime)s: %(levelname)s/%(processName)s] %(message)s'
worker_task_log_format = '[%(asctime)s: %(levelname)s/%(processName)s] [%(task_name)s(%(task_id)s)] %(message)s'

# 性能优化
worker_disable_rate_limits = True  # 禁用任务频率限制
task_acks_late = True  # 任务执行完成后再确认
task_reject_on_worker_lost = True  # worker异常关闭时任务会被重新分配

# 任务执行配置
task_always_eager = False  # 是否立即执行任务（调试用）


@shared_task(
    bind=True,
    base=RetryableTask,
)
def complex_task(self, *args, **kwargs):
    try:
        # 任务逻辑
        result = process_data()
        return result
    except TemporaryError as exc:
        # 临时错误，使用指数退避重试
        retry_countdown = min(2 ** self.request.retries * 60, 3600)
        raise self.retry(exc=exc, countdown=retry_countdown)
    except PermanentError as exc:
        # 永久错误，记录并标记失败
        logger.error(f"Permanent error occurred: {exc}")
        raise



#!/usr/bin/env python

"""Tests for `{{ cookiecutter.project_slug }}` package."""

{% if cookiecutter.use_pytest == 'y' -%}
import pytest
{% else %}
import unittest
{%- endif %}
{%- if cookiecutter.command_line_interface|lower == 'click' %}
from click.testing import CliRunner
{%- endif %}

from {{ cookiecutter.project_slug }} import {{ cookiecutter.project_slug }}
{%- if cookiecutter.command_line_interface|lower == 'click' %}
from {{ cookiecutter.project_slug }} import cli
{%- endif %}

{%- if cookiecutter.use_pytest == 'y' %}


@pytest.fixture
def response():
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """
    # import requests
    # return requests.get('https://github.com/audreyr/cookiecutter-pypackage')


def test_content(response):
    """Sample pytest test function with the pytest fixture as an argument."""
    # from bs4 import BeautifulSoup
    # assert 'GitHub' in BeautifulSoup(response.content).title.string
{%- if cookiecutter.command_line_interface|lower == 'click' %}


def test_command_line_interface():
    """Test the CLI."""
    runner = CliRunner()
    result = runner.invoke(cli.main)
    assert result.exit_code == 0
    assert '{{ cookiecutter.project_slug }}.cli.main' in result.output
    help_result = runner.invoke(cli.main, ['--help'])
    assert help_result.exit_code == 0
    assert '--help  Show this message and exit.' in help_result.output
{%- endif %}
{%- else %}


class Test{{ cookiecutter.project_slug|title }}(unittest.TestCase):
    """Tests for `{{ cookiecutter.project_slug }}` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_something(self):
        """Test something."""
{%- if cookiecutter.command_line_interface|lower == 'click' %}

    def test_command_line_interface(self):
        """Test the CLI."""
        runner = CliRunner()
        result = runner.invoke(cli.main)
        assert result.exit_code == 0
        assert '{{ cookiecutter.project_slug }}.cli.main' in result.output
        help_result = runner.invoke(cli.main, ['--help'])
        assert help_result.exit_code == 0
        assert '--help  Show this message and exit.' in help_result.output
{%- endif %}
{%- endif %}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : json_utils
# @Time         : 2021/4/22 1:51 下午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : pd.io.json.json_normalize

# https://mangiucugna.github.io/json_repair/
# https://jsonpath.com/

import jsonpath

# json https://blog.csdn.net/freeking101/article/details/103048514
# https://github.com/ijl/orjson#quickstart
# https://jmespath.org/tutorial.html
# https://goessner.net/articles/JsonPath/
# https://www.jianshu.com/p/3f5b9cc88bde

# todo: jsonpath jmespath
# https://blog.csdn.net/be5yond/article/details/118976017
# https://blog.csdn.net/weixin_44799217/article/details/127590589

from meutils.pipe import *
from json_repair import repair_json as _repair_json


@lru_cache()
def repair_json(json_str, **kwargs):
    return _repair_json(json_str, **kwargs)


def json2class(dic, class_name='Test'):
    s = f"""class {class_name}(BaseModel):"""
    for k, v in dic.items():
        _type = type(v).__name__
        if isinstance(_type, str):
            v = f"'{v}'"
        s += f"\n\t{k}: {_type} = {v}"

    print(s)


@lru_cache(1024)
def json_loads(s):
    if isinstance(s, bytes):
        s = s.decode()
    try:
        return json.loads(s.replace("'", '"'))

    except Exception as e:
        logger.warning(e)

        return eval(s)


def json_path(obj, expr):  # todo: 缓存
    """
    $..["keywords","query","search_result"]

    python =>     $..[keywords,query,search_result]

    """
    if isinstance(obj, dict):
        pass
    elif isinstance(obj, str):
        obj = json_repair.loads(obj)
    elif isinstance(obj, bytes):
        obj = json_repair.loads(obj.decode())
    elif isinstance(obj, BaseModel):
        obj = obj.dict()

    return jsonpath.jsonpath(obj, expr=expr)


if __name__ == '__main__':
    # print(json_path({"a": 1}, expr='$.a'))
    # print(json_path("""{"a": 1}""", expr='$.a'))
    #
    # json_string = """{"a": 1}"""

    class A(BaseModel):
        a: int = 1


    data = {
        "id": "cgt-20250613173405-qnpqg",
        "model": "doubao-seedance-1-0-pro-250528",
        "status": "succeeded",
        "content": {
            "video_url": "https://ark-content-generation-cn-beijing.tos-cn-beijing.volces.com/doubao-seedance-1-0-pro/02174980724664200000000000000000000ffffac182c177b9d12.mp4?X-Tos-Algorithm=TOS4-HMAC-SHA256&X-Tos-Credential=AKLTYjg3ZjNlOGM0YzQyNGE1MmI2MDFiOTM3Y2IwMTY3OTE%2F20250613%2Fcn-beijing%2Ftos%2Frequest&X-Tos-Date=20250613T093454Z&X-Tos-Expires=86400&X-Tos-Signature=bc080dc9e02282dbe10c82e04c59ac1ed4afb67cbec8aa0506357540f9d47fc4&X-Tos-SignedHeaders=host"
        },
        "usage": {
            "completion_tokens": 245388,
            "total_tokens": 245388
        },
        "created_at": 1749807246,
        "updated_at": 1749807294
    }

    # print(json_path(data, expr='$..[url,image_url,video_url]'))

    # print(repair_json('{a: 1}', return_objects=True))
    # print(type(repair_json('{a: 1}', return_objects=True)))
    # print(repair_json('{\\"thinking\\":{\\"type\\":\\"enabled\\"}}', return_objects=True)) # 错

    data =  {
        "data": [
            {
                "taskType": "videoInference",
                "taskUUID": "8a5a1c09-d0a5-4b1b-9b67-8943cacc935f"
            }
        ]
    }


    print(json_path(data, expr='$..taskUUID'))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : video_types
# @Time         : 2024/9/13 10:15
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import uuid

from meutils.pipe import *

from openai.types.video import Video as _Video


class Video(_Video):
    id: str = Field(default_factory=lambda: str(uuid.uuid4()))
    """Unique identifier for the video job."""

    created_at: int = Field(default_factory=lambda: int(time.time()))
    """Unix timestamp (seconds) for when the job was created."""

    model: Optional[Union[str, Literal["sora-2", "sora-2-pro"]]] = None
    """The video generation model that produced the job."""

    object: Literal["video"] = "video"
    """The object type, which is always `video`."""

    progress: int = 0
    """Approximate completion percentage for the generation task."""

    seconds: Optional[Union[str, Literal["4", "8", "12"]]] = None
    """Duration of the generated clip in seconds."""

    size: Optional[Union[str, Literal["720x1280", "1280x720", "1024x1792", "1792x1024"]]] = None
    """The resolution of the generated video."""

    status: Literal["queued", "in_progress", "completed", "failed"] = "queued"
    """Current lifecycle status of the video job."""

    video_url: Optional[str] = None

    # class Config:
    #     extra = "allow"


class VideoRequest(BaseModel):
    model: Union[str, Literal["cogvideox-flash", "cogvideox"]] = "cogvideox-flash"

    prompt: str = "比得兔开小汽车，游走在马路上，脸上的表情充满开心喜悦。"
    negative_prompt: Optional[str] = None

    """
    提供基于其生成内容的图像。如果传入此参数，系统将以该图像为基础进行操作。支持通过URL或Base64编码传入图片。
    图片要求如下：图片支持.png、jpeg、.jpg 格式、图片大小：不超过5M。image_url和prompt二选一或者同时传入。
    """
    image_url: Optional[str] = None
    tail_image_url: Optional[str] = None

    """
    输出模式，默认为 "quality"。 "quality"：质量优先，生成质量高。 "speed"：速度优先，生成时间更快，质量相对降低。 
    cogvideox-flash模型不支持选择输出模式。
    """
    quality: Literal["quality", "speed"] = "speed"

    """是否生成 AI 音效。默认值: False（不生成音效）。"""
    with_audio: bool = True

    cfg_scale: Optional[float] = None

    """
    默认值: 若不指定，默认生成视频的短边为 1080，长边根据原图片比例缩放。最高支持 4K 分辨率。
    分辨率选项：720x480、1024x1024、1280x960、960x1280、1920x1080、1080x1920、2048x1080、3840x2160
    """
    aspect_ratio: Union[str, Literal["1:1", "21:9", "16:9", "9:16", "4:3", "3:4"]] = "16:9"

    size: Literal[
        '720x480',
        '1024x1024',
        '1280x960',
        '960x1280',
        '1920x1080',
        '1080x1920',
        '2048x1080',
        '3840x2160'] = "1024x1024"

    duration: Literal[5, 10] = 5

    fps: Literal[30, 60] = 30


class SoraVideoRequest(BaseModel):
    model: Union[str, Literal["sora-2", "sora-2-pro"]] = "sora-2"
    prompt: str = "比得兔开小汽车，游走在马路上，脸上的表情充满开心喜悦。"
    seconds: Union[str, Literal["4", "8", "12"]] = "4"
    size: Optional[Union[str, Literal["720x1280", "1280x720", "1024x1792", "1792x1024"]]] = "720x1280"
    input_reference: Optional[Union[str, bytes, list]] = None  # image url/base64/bytes
    webhook_url: Optional[str] = None


class FalVideoRequest(BaseModel):
    model: Union[str, Literal["latentsync", "sync-lipsync",]] = 'latentsync'
    video_url: Optional[str] = None
    audio_url: Optional[str] = None
    image_url: Optional[str] = None

    sync_mode: Union[str, Literal["cut_off", "loop", "bounce"]] = "cut_off"


class FalKlingVideoRequest(BaseModel):
    model: Union[
        str, Literal["fal-ai/kling-video/v1/standard/text-to-video",]] = 'fal-ai/kling-video/v1/standard/text-to-video'

    prompt: Optional[str] = None
    duration: Optional[float] = 5.0
    video_url: Optional[str] = None
    audio_url: Optional[str] = None
    image_url: Optional[str] = None

    sync_mode: Union[str, Literal["cut_off", "loop", "bounce"]] = "cut_off"


class LipsyncVideoRequest(BaseModel):
    model: Union[str, Literal[
        "latentsync", "sync-lipsync",
        "lip_sync_avatar_std", "lip_sync_avatar_lively"
    ]
    ] = 'latentsync'

    video_url: Optional[str] = None
    audio_url: Optional[str] = None
    image_url: Optional[str] = None

    sync_mode: Union[str, Literal["cut_off", "loop", "bounce"]] = "cut_off"


if __name__ == '__main__':
    # print(LipsyncVideoRequest())

    print(Video(x=1))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : models
# @Time         : 2025/4/14 11:09
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *
from meutils.llm.clients import OpenAI, AsyncOpenAI


async def check_token(api_key):
    try:
        client = AsyncOpenAI(
            base_url=os.getenv("OPENROUTER_BASE_URL"),
            api_key=api_key,
        )
        r = await client.chat.completions.create(
            messages=[{'role': 'user', 'content': 'hi'}],
            model="google/gemma-3-1b-it:free",
            max_tokens=1,
            stream=False
        )
        logger.debug(r)
        return True

    except Exception as e:
        logger.error(e)
        return False


async def check_token_for_together(api_key):
    model = "meta-llama/Llama-Vision-Free"
    try:
        client = AsyncOpenAI(
            base_url=os.getenv("TOGETHER_BASE_URL"),
            api_key=api_key,
        )
        r = await client.chat.completions.create(
            messages=[{'role': 'user', 'content': 'hi'}],
            model=model,
            max_tokens=1,
            stream=False
        )
        logger.debug(r)
        return True

    except Exception as e:
        logger.error(e)
        return False


def get_openrouter_models():
    models = OpenAI(base_url=os.getenv("OPENROUTER_BASE_URL"), api_key='xx').models.list()

    data = {}
    for model in models.data:
        if model.id.lower().endswith(':free'):
            _model = model.id.lower().removesuffix(":free").split('/')[-1]
            data[_model] = f"""{_model}=={model.id}"""

    print(data | xjoin(","))
    return data


def get_together_models():
    client = OpenAI(base_url=os.getenv("TOGETHER_BASE_URL"), api_key=os.getenv("TOGETHER_API_KEY"))
    models = client.get("models", cast_to=object)
    # logger.debug(bjson(models))

    data = {}
    for model in models:
        if model['id'].lower().endswith('-free'):
            _model = model['id'].lower().removesuffix("-free").split('/')[-1]
            data[_model] = f"""{_model}=={model['id']}"""

    print(data | xjoin(","))
    return data


if __name__ == '__main__':
    from meutils.config_utils.lark_utils import get_series

    # print(bjson(get_openrouter_models()))
    # print(bjson(get_together_models()))

    # arun(check_token("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"))

    # tokens = arun(get_series("https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=gGFIXb"))
    #
    # r = []
    # for i in tokens:
    #     if not arun(check_token(i)):
    #         print(i)
    #         r.append(i)

    feishu_url = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=tEsIyw"

    tokens = arun(get_series(feishu_url))

    r = []
    rr = []
    for i in tokens:
        if not arun(check_token_for_together(i)):
            print(i)
            r.append(i)
        else:
            rr.append(i)

    # arun(check_token_for_together("1581bb1c605501c96569cf9a24aafa7361752697a23475cdf8f2c3fe8a488292"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : tts
# @Time         : 2023/10/25 18:21
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://www.qianduan.shop/blogs/detail/140
# https://baijiahao.baidu.com/s?id=1762431177147996323&wfr=spider&for=pc
# https://github.com/pndurette/gTTS
# https://github.com/ycyy/edge-tts-webui

from meutils.pipe import *
import gradio as gr
import edge_tts
import asyncio
import os

# https://speech.platform.bing.com/consumer/speech/synthesize/readaloud/voices/list?trustedclienttoken=6A5AA1D4EAFF4E9FB37E23D68491D6F4
SUPPORTED_VOICES = {
    'Xiaoxiao-晓晓': 'zh-CN-XiaoxiaoNeural',
    'Xiaoyi-晓伊': 'zh-CN-XiaoyiNeural',
    'Yunjian-云健': 'zh-CN-YunjianNeural',
    'Yunxi-云希': 'zh-CN-YunxiNeural',
    'Yunxia-云夏': 'zh-CN-YunxiaNeural',
    'Yunyang-云扬': 'zh-CN-YunyangNeural',
    'liaoning-Xiaobei-晓北辽宁': 'zh-CN-liaoning-XiaobeiNeural',
    'shaanxi-Xiaoni-陕西晓妮': 'zh-CN-shaanxi-XiaoniNeural'
}


# 发音切换
def changeVoice(voices):
    example = SUPPORTED_VOICES[voices]
    example_file = os.path.join(os.path.dirname(__file__), "example/" + example + ".wav")
    return example_file


# 文本转语音
async def textToSpeech(text, voices, rate, volume):
    output_file = "output.mp3"
    voices = SUPPORTED_VOICES[voices]
    if (rate >= 0):
        rates = rate = "+" + str(rate) + "%"
    else:
        rates = str(rate) + "%"
    if (volume >= 0):
        volumes = "+" + str(volume) + "%"
    else:
        volumes = str(volume) + "%"
    communicate = edge_tts.Communicate(text,
                                       voices,
                                       rate=rates,
                                       volume=volumes,
                                       proxy=None)
    await communicate.save(output_file)
    audio_file = os.path.join(os.path.dirname(__file__), "output.mp3")
    if (os.path.exists(audio_file)):
        return audio_file
    else:
        raise gr.Error("转换失败！")
        return FileNotFoundError


# 清除转换结果
def clearSpeech():
    output_file = os.path.join(os.path.dirname(__file__), "output.mp3")
    if (os.path.exists(output_file)):
        os.remove(output_file)
    return None, None


with gr.Blocks(css="style.css", title="文本转语音") as demo:
    gr.Markdown("""
    # 🔥ChatLLM 语音合成系统`TTS`
    """)
    with gr.Row():
        with gr.Column():
            text = gr.TextArea(label="文本", elem_classes="text-area")
            btn = gr.Button("生成", elem_id="submit-btn")
        with gr.Column():
            voices = gr.Dropdown(choices=[
                "Xiaoxiao-晓晓", "Xiaoyi-晓伊", "Yunjian-云健", "Yunxi-云希",
                "Yunxia-云夏", "Yunyang-云扬", "liaoning-Xiaobei-晓北辽宁",
                "shaanxi-Xiaoni-陕西晓妮"
            ],
                value="Xiaoxiao-晓晓",
                label="发音",
                info="请选择发音人",
                interactive=True)

            example = gr.Audio(label="试听",
                               value="example/zh-CN-XiaoxiaoNeural.wav",
                               interactive=False,
                               elem_classes="example")

            voices.change(fn=changeVoice, inputs=voices, outputs=example)
            rate = gr.Slider(-100,
                             100,
                             step=1,
                             value=0,
                             label="语速增减",
                             info="加快或减慢语速",
                             interactive=True)

            volume = gr.Slider(-100,
                               100,
                               step=1,
                               value=0,
                               label="音调增减",
                               info="加大或减小音调",
                               interactive=True)
            audio = gr.Audio(label="输出",
                             interactive=False,
                             elem_classes="audio")
            clear = gr.Button("清除", elem_id="clear-btn")
            btn.click(fn=textToSpeech,
                      inputs=[text, voices, rate, volume],
                      outputs=[audio])
            clear.click(fn=clearSpeech, outputs=[text, audio])

if __name__ == "__main__":
    import uvicorn
    from fastapi import FastAPI

    app = FastAPI()

    app = gr.mount_gradio_app(app, demo, path="/web/tts")
    uvicorn.run(app, host="0.0.0.0", port=39999)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : echarts
# @Time         : 2022/5/16 下午1:38
# @Author       : yuanjie
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

from pyecharts import options as opts
from pyecharts.charts import WordCloud
from pyecharts.globals import SymbolType  # Symbol 样式类型,见“基本使用”中的“全局变量”介绍
from pyecharts.globals import RenderType  # 渲染方式


fk = Faker('zh')
words = data = [(fk.name(), fk.random_int()) for i in range(1000)]


def wc_plot(data):
    init_opts = opts.InitOpts(
        # 图表画布宽度，css 长度单位。
        width="900px",

        # 图表画布高度，css 长度单位。
        height="500px",

        # 图表 ID，图表唯一标识，用于在多图表时区分。
        chart_id=None,

        # 渲染风格，可选 "canvas", "svg"
        # # 参考 `全局变量` 章节
        renderer=RenderType.CANVAS,

        # 网页标题
        page_title="Wordcloud_custom_font_style",

        # 图表主题
        theme="white",

        # 图表背景颜色
        bg_color=None,

        # 远程 js host，如不设置默认为 https://assets.pyecharts.org/assets/"
        # 参考 `全局变量` 章节
        js_host="",

        # 画图动画初始化配置，参考 `global_options.AnimationOpts`
        animation_opts=opts.AnimationOpts(),
    )
    return (
        WordCloud(init_opts=init_opts)
            .add("",  # 系列名称
                 words,  # 系列数据项，[(word1, count1), (word2, count2)]

                 # shape=SymbolType.DIAMOND,
                 shape="star",
                 # 词云图轮廓，有 'circle', 'cardioid', 'diamond', 'triangle-forward', 'triangle', 'pentagon', 'star' 可选

                 # 单词间隔
                 word_gap=20,

                 # 旋转单词角度
                 rotate_step=45,

                 # 单词字体大小范围
                 word_size_range=[20, 100],

                 textstyle_opts=opts.TextStyleOpts(font_family="cursive"),
                 )
            .set_global_opts(
            title_opts=opts.TitleOpts(title="WordCloud-自定义文字样式", title_textstyle_opts=opts.TextStyleOpts(font_size=23)),
            tooltip_opts=opts.TooltipOpts(is_show=True)  # 是否显示提示框组件，包括提示框浮层和 axisPointer。
        )
            .render("wordcloud_custom_font_style_2.html")  # .render_notebook()
    )


if __name__ == '__main__':
    wc_plot(words)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : __init__.py
# @Time         : 2025/9/3 14:40
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

def main(arg1: dict):
    if docs := arg1:
        contents = []
        for i, doc in enumerate(docs, 1):
            _ = f"""
            [{i}] {doc.get("title")}

            {doc.get("content")}
            """
            contents.append(_)
        return {
            "result": "\n\n\n".join(contents)
        }

def main(arg1: list):
    docs = arg1

    _docs = []
    for i, doc in enumerate(docs, 1):

        doc['content'] = f"""
        [{i}] {doc.get("title")}

        {doc.get("content")}
        """.strip()
        _docs.append(doc)

    return {
        "result": _docs
    }


if __name__ == '__main__':
    print(main([]))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : channel
# @Time         : 2024/10/9 18:53
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.str_utils.json_utils import json_path
from meutils.hash_utils import murmurhash
from meutils.schemas.oneapi import BASE_URL, GROUP_RATIO
from meutils.schemas.oneapi._types import ChannelInfo
from meutils.schemas.db.oneapi_types import OneapiChannel

from meutils.db.orm import update_or_insert

headers = {
    'authorization': f'Bearer {os.getenv("CHATFIRE_ONEAPI_TOKEN")}',
    'new-api-user': '1',
    'rix-api-user': '1',
}

filter_kwargs = {
    # "key": "gpoH1z3G6nHovD8MY40i6xx5tsC1vbh7B3Aao2jmejYNoKhv",
    # "key": "610d41b8-0b6e-4fba-8439-f5178b733f3a",
    "id": 21249,
}


# async def update_channel(id: int, base_url: str):
#     arun(update_or_insert(OneapiChannel, filter_kwargs, update_fn))
#
#     filter_kwargs = {
#         "id": "1",
#     }
#
#     async def update_fn(data):
#         data.key = "k1\nk2\nk3"
#         return data
#
#     async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=30) as client:
#         response = await client.get(f"/api/channel/{id}")
#         response.raise_for_status()
#
#         data = response.json()
#         logger.debug(bjson(data))
#         return data


async def manage_multi_key(id: int, base_url: str, action: str = "enable_all_keys") -> ChannelInfo:
    # "disable_all_keys"
    payload = {
        "channel_id": id,
        "action": action
    }  #
    async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=30) as client:
        response = await client.post("/api/channel/multi_key/manage", json=payload)
        response.raise_for_status()

        data = response.json()
        logger.debug(bjson(data))
        return data


async def get_channel_info(id: int, base_url: str, response_format: Optional[str] = None):
    params = {"keyword": id}  # keyword=21222&group=&model=&id_sort=true&tag_mode=false&p=1&page_size=100

    async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=30) as client:
        response = await client.get("/api/channel/search", params=params)
        response.raise_for_status()

        data = response.json()
        logger.debug(bjson(data))

        if (items := data.get("data", {}).get("items", [])) and (item := items[0]):
            if not response_format: return item

            if response_format == "status":  # multi_key_status
                return item.get("status") == 1  # status==1 是正常

            elif item.get("status") == 1 and not item.get("channel_info", {}).get("multi_key_status_list"):
                logger.debug(f"渠道 {id} 渠道全key正常")
                return True


async def edit_channel(models, token: Optional[str] = None):
    token = token or os.environ.get("CHATFIRE_ONEAPI_TOKEN")

    models = ','.join(filter(lambda model: model.startswith(("api", "official-api", "ppu", "kling-v")), models))
    models += ",suno-v3,indextts-1.5,cosyvoice2,step-audio-tts-3b,f5-tts"

    payload = {
        "id": 289,
        "type": 1,
        "key": "",
        "openai_organization": "",
        "test_model": "ppu",
        "status": 1,
        "name": "按次收费ppu",
        "weight": 0,
        "created_time": 1717038002,
        "test_time": 1728212103,
        "response_time": 9,
        "base_url": "https://ppu.chatfire.cn",
        "other": "",
        "balance": 0,
        "balance_updated_time": 1726793323,
        "models": models,
        "used_quota": 4220352321,
        "model_mapping": "",
        "status_code_mapping": "",
        "priority": 1,
        "auto_ban": 0,
        "other_info": "",

        "group": "default,openai,chatfire,enterprise",  # ','.join(GROUP_RATIO),
        "groups": ['default']
    }
    headers = {
        'authorization': f'Bearer {token}',
        'rix-api-user': '1'
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=30) as client:
        response = await client.put("/api/channel/", json=payload)
        response.raise_for_status()
        logger.debug(bjson(response.json()))

        payload['id'] = 280
        payload['name'] = '按次收费ppu-cc'
        payload['priority'] = 0
        payload['base_url'] = 'https://ppu.chatfire.cc'

        response = await client.put("/api/channel/", json=payload)
        response.raise_for_status()
        logger.debug(bjson(response.json()))


async def exist_channel(
        request: ChannelInfo,

        base_url: Optional[str] = None,
        api_key: Optional[str] = None,
):
    if request.id is None:
        return False
    base_url = base_url or "https://api.chatfire.cn"
    api_key = api_key or os.getenv("CHATFIRE_ONEAPI_TOKEN")

    headers = {
        'authorization': f'Bearer {api_key}',
        'new-api-user': '1',
        'rix-api-user': '1',
    }
    async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=100) as client:
        if "api.chatfire.cn" in base_url:
            path = "/api/channel/"
            params = {"channel_id": request.id}
        else:
            path = "/api/channel/search"
            params = {"keyword": request.id}
        response = await client.get(path, params=params)
        response.raise_for_status()

        logger.debug(bjson(response.json()))

        if items := response.json()['data']['items']:
            _ = [item for item in items if item['id'] == request.id]
            # logger.debug(_)

            return _
        else:
            logger.debug(f"渠道不存在：{request.id}")
            return False


async def create_or_update_channel(
        request: ChannelInfo,

        base_url: Optional[str] = None,
        api_key: Optional[str] = None,
):
    assert base_url, "base_url 不能为空"

    if request.id and isinstance(request.id, str):  # 批量更新
        ids = []
        if "," in request.id:
            ids = map(int, request.id.split(","))
        elif ':' in request.id:
            start, end = map(int, request.id.split(":"))
            ids = range(start, end)

        request_list = []
        for i, k in zip(ids, request.key.split()):  # api_key不为空, 如果id很多是否考虑复制 api_key不为空
            _request = request.copy()
            _request.id = i
            _request.key = k
            request_list.append(_request)

        tasks = [create_or_update_channel(r, base_url, api_key) for r in request_list]
        return await asyncio.gather(*tasks)

    method = "post"
    if await exist_channel(request, base_url, api_key):
        logger.debug(f"渠道已存在，跳过创建：{request.id}")
        method = "put"

    # 新创建的优先级低，保证旧key刷的时间更长
    request.priority = request.priority or int(1000 - (time.time() - time.time() // 1000 * 1000))

    api_key = api_key or os.getenv("CHATFIRE_ONEAPI_TOKEN")
    headers = {
        'authorization': f'Bearer {api_key}',
        'new-api-user': '1',
        'rix-api-user': '1',
    }
    payload = request.model_dump(exclude_none=True)
    if "api.chatfire.cn" not in base_url and method == "post":
        payload = {
            "mode": "single",
            "channel": payload
        }

    async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=100) as client:
        response = await client.request(method, "/api/channel/", json=payload)
        # logger.debug(response.text)
        response.raise_for_status()

        data = response.json()
        logger.debug(bjson(data))
        if (data := data.get("data")) and data['channel_info']['is_multi_key']:
            await manage_multi_key(data['id'], base_url=base_url)

        return response.json()


async def create_or_update_channel_for_gemini(api_key, base_url: Optional[str] = "https://api.ffire.cc"):
    if isinstance(api_key, list):
        api_keys = api_key | xgroup(128)  # [[],]
    else:
        api_keys = [[api_key]]

    models = "gemini-2.5-flash-preview-05-20,gemini-1.5-flash-latest,gemini-1.5-flash-001,gemini-1.5-flash-001-tuning,gemini-1.5-flash,gemini-1.5-flash-002,gemini-1.5-flash-8b,gemini-1.5-flash-8b-001,gemini-1.5-flash-8b-latest,gemini-1.5-flash-8b-exp-0827,gemini-1.5-flash-8b-exp-0924,gemini-2.5-flash-preview-04-17,gemini-2.0-flash-exp,gemini-2.0-flash,gemini-2.0-flash-001,gemini-2.0-flash-exp-image-generation,gemini-2.0-flash-lite-001,gemini-2.0-flash-lite,gemini-2.0-flash-lite-preview-02-05,gemini-2.0-flash-lite-preview,gemini-2.0-flash-thinking-exp-01-21,gemini-2.0-flash-thinking-exp,gemini-2.0-flash-thinking-exp-1219,learnlm-2.0-flash-experimental,gemma-3-1b-it,gemma-3-4b-it,gemma-3-12b-it,gemma-3-27b-it,gemini-2.0-flash-live-001"
    nothinking_models = 'gemini-2.5-flash-preview-05-20-nothinking,gemini-2.5-flash-preview-04-17-nothinking,gemini-2.0-flash-thinking-exp-01-21-nothinking,gemini-2.0-flash-thinking-exp-nothinking,gemini-2.0-flash-thinking-exp-1219-nothinking'
    models = f"{models},{nothinking_models}"

    payload = {
        # "id": 7493,
        "type": 24,  # gemini
        # "key": "AIzaSyCXWV19FRM4XX0KHmpR9lYUz9i1wxQTYUg",
        "openai_organization": "",
        "test_model": "",
        "status": 1,
        "name": "gemini",

        "priority": murmurhash(api_key, bins=3),
        "weight": 0,
        # "created_time": 1745554162,
        # "test_time": 1745554168,
        # "response_time": 575,
        # "base_url": "https://g.chatfire.cn/v1beta/openai/chat/completions",
        # "other": "",
        # "balance": 0,
        # "balance_updated_time": 0,
        "models": models,
        # "used_quota": 0,
        "model_mapping": """{"gemini-2.5-pro-preview-03-25": "gemini-2.5-pro-exp-03-25"}""",
        # "status_code_mapping": "",
        # "auto_ban": 1,
        # "other_info": "",
        # "settings": "",
        "tag": "gemini",
        # "setting": None,
        # "param_override": "\n {\n \"seed\": null,\n \"frequency_penalty\": null,\n \"presence_penalty\": null,\n \"max_tokens\": null\n }\n ",
        "group": "default",
        "groups": [
            "default"
        ]
    }

    for api_key in tqdm(api_keys):
        payload['key'] = '\n'.join(api_key)
        # logger.debug(payload)
        async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=100) as client:
            response = await client.post("/api/channel/", json=payload)
            response.raise_for_status()
            logger.debug(response.json())


async def delete_channel(id, base_url: Optional[str] = "https://api.ffire.cc"):
    ids = id
    if isinstance(id, str):
        ids = [id]

    for _ids in tqdm(ids | xgroup(256)):
        payload = {
            "ids": list(_ids)
        }
        async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=100) as client:
            response = await client.post(f"/api/channel/batch", json=payload)
            response.raise_for_status()
            logger.debug(response.json())


async def update_channel(id: int, base_url: str, **kwargs):
    """只更新keys"""

    # key_mode: Optional[Literal['append', 'replace']] = None  # none是覆盖

    if channel_info := await get_channel_info(id, base_url):
        channel_info = {**channel_info, **kwargs}

        async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=100) as client:
            response = await client.request("PUT", "/api/channel/", json=channel_info)
            response.raise_for_status()

            return response.json()


if __name__ == '__main__':
    from meutils.config_utils.lark_utils import get_series

    # models = "gemini-1.0-pro-vision-latest,gemini-pro-vision,gemini-1.5-pro-latest,gemini-1.5-pro-001,gemini-1.5-pro-002,gemini-1.5-pro,gemini-1.5-flash-latest,gemini-1.5-flash-001,gemini-1.5-flash-001-tuning,gemini-1.5-flash,gemini-1.5-flash-002,gemini-1.5-flash-8b,gemini-1.5-flash-8b-001,gemini-1.5-flash-8b-latest,gemini-1.5-flash-8b-exp-0827,gemini-1.5-flash-8b-exp-0924,gemini-2.5-pro-exp-03-25,gemini-2.5-pro-preview-03-25,gemini-2.5-flash-preview-04-17,gemini-2.0-flash-exp,gemini-2.0-flash,gemini-2.0-flash-001,gemini-2.0-flash-exp-image-generation,gemini-2.0-flash-lite-001,gemini-2.0-flash-lite,gemini-2.0-flash-lite-preview-02-05,gemini-2.0-flash-lite-preview,gemini-2.0-pro-exp,gemini-2.0-pro-exp-02-05,gemini-2.0-flash-thinking-exp-01-21,gemini-2.0-flash-thinking-exp,gemini-2.0-flash-thinking-exp-1219,learnlm-1.5-pro-experimental,learnlm-2.0-flash-experimental,gemma-3-1b-it,gemma-3-4b-it,gemma-3-12b-it,gemma-3-27b-it,gemini-2.0-flash-live-001"
    # nothinking_models = [f"{model}-nothinking" for model in models.split(',') if
    #                      (model.startswith('gemini-2.5') or "thinking" in model)] | xjoin(',')
    #
    # nothinking_models = 'gemini-2.5-pro-exp-03-25-nothinking,gemini-2.5-pro-preview-03-25-nothinking,gemini-2.5-flash-preview-04-17-nothinking,gemini-2.0-flash-thinking-exp-01-21-nothinking,gemini-2.0-flash-thinking-exp-nothinking,gemini-2.0-flash-thinking-exp-1219-nothinking'

    # gemini
    FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=kfKGzt"
    #
    base_url = "https://api.ffire.cc"
    # base_url = "https://usa.chatfire.cn"
    base_url = "https://api.chatfire.cn"

    # tokens = arun(get_series(FEISHU_URL))  # [:5]
    # arun(create_or_update_channel(tokens, base_url))
    # arun(create_or_update_channel(tokens))
    # # arun(delete_channel(range(10000, 20000)))
    key = "KEY\nKEY2\nKEY2\nKEY2\nKEY2"
    # request = ChannelInfo(id = 1, name='xx', key=key)
    # request = ChannelInfo(id=21223, key=key, used_quota=0.001)
    request = ChannelInfo(id=29483, key=key, used_quota=0.001,
                          # key_mode="append"
                          )

    request = ChannelInfo(id=29499, key=key, type=45)

    # arun(create_or_update_channel(request, base_url=base_url))
    #
    # arun(exist_channel(request, base_url=base_url))

    # arun(get_channel_info(21222, base_url=base_url))
    # arun(get_channel_info(29495, base_url=base_url, response_format="status"))
    # arun(get_channel_info(29495, base_url=base_url, response_format="multi_key_status"))

    # arun(delete_channel(range(10015, 10032), base_url=base_url))

    # arun(manage_multi_key(29499, base_url=base_url))

    arun(update_channel(21249, base_url=base_url, key=key, key_mode="append"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : demo
# @Time         : 2021/9/4 下午3:26
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  :


